@@ -15,10 +15,11 @@ import (
 
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/internal/command"
-	"gitlab.com/gitlab-org/gitaly/internal/config"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
+	gitalyhook "gitlab.com/gitlab-org/gitaly/internal/gitaly/hook"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/service/hook"
 	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/metadata"
-	"gitlab.com/gitlab-org/gitaly/internal/service/hook"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc/reflection"
@@ -49,6 +50,94 @@ func testMain(m *testing.M) int {
 	return m.Run()
 }
 
+func TestReftxHookFixup(t *testing.T) {
+	testcases := []struct {
+		desc  string
+		args  []string
+		fixed bool
+	}{
+		{
+			desc:  "empty arguments",
+			args:  []string{},
+			fixed: false,
+		},
+		{
+			desc:  "single argument",
+			args:  []string{"gitaly-hooks"},
+			fixed: false,
+		},
+		{
+			desc:  "check",
+			args:  []string{"gitaly-hooks", "check"},
+			fixed: false,
+		},
+		{
+			desc:  "check with arguments",
+			args:  []string{"gitaly-hooks", "check", "foo", "bar"},
+			fixed: false,
+		},
+		{
+			desc:  "pre-receive hook without arguments",
+			args:  []string{"gitaly-hooks", "pre-receive"},
+			fixed: false,
+		},
+		{
+			desc:  "post-receive hook without arguments",
+			args:  []string{"gitaly-hooks", "post-receive"},
+			fixed: false,
+		},
+		{
+			desc:  "update hook without arguments",
+			args:  []string{"gitaly-hooks", "update"},
+			fixed: false,
+		},
+		{
+			desc:  "update with typical arguments",
+			args:  []string{"gitaly-hooks", "update", "refs/heads/master", "foo", "bar"},
+			fixed: false,
+		},
+		{
+			desc:  "update with prepared argument",
+			args:  []string{"gitaly-hooks", "update", "prepared", "foo"},
+			fixed: false,
+		},
+		{
+			desc:  "wrongly called update hook",
+			args:  []string{"gitaly-hooks", "update", "prepared"},
+			fixed: true,
+		},
+		{
+			desc:  "wrongly called pre-receive hook",
+			args:  []string{"gitaly-hooks", "pre-receive", "committed"},
+			fixed: true,
+		},
+		{
+			desc:  "wrongly called post-receive hook",
+			args:  []string{"gitaly-hooks", "post-receive", "aborted"},
+			fixed: true,
+		},
+		{
+			desc:  "unrelated hook",
+			args:  []string{"gitaly-hooks", "pre-commit", "prepared"},
+			fixed: false,
+		},
+	}
+
+	for _, tc := range testcases {
+		t.Run(tc.desc, func(t *testing.T) {
+			actual := fixupReftxHook(tc.args)
+
+			expected := make([]string, len(tc.args))
+			copy(expected, tc.args)
+			if tc.fixed {
+				expected[1] = "reference-transaction"
+			}
+
+			require.Equal(t, actual, expected)
+		})
+	}
+}
+
 func TestHooksPrePostWithSymlinkedStoragePath(t *testing.T) {
 	tempDir, cleanup := testhelper.TempDir(t)
 	defer cleanup()
@@ -110,15 +199,15 @@ func testHooksPrePostReceive(t *testing.T) {
 		RepoPath:                    testRepoPath,
 	}
 
-	ts := testhelper.NewGitlabTestServer(c)
-	defer ts.Close()
+	serverURL, cleanup := testhelper.NewGitlabTestServer(t, c)
+	defer cleanup()
 
 	config.Config.GitlabShell.Dir = tempGitlabShellDir
 
 	testhelper.WriteTemporaryGitlabShellConfigFile(t,
 		tempGitlabShellDir,
 		testhelper.GitlabShellConfig{
-			GitlabURL: ts.URL,
+			GitlabURL: serverURL,
 			HTTPSettings: testhelper.HTTPSettings{
 				User:     gitlabUser,
 				Password: gitlabPassword,
@@ -127,7 +216,7 @@ func testHooksPrePostReceive(t *testing.T) {
 
 	testhelper.WriteShellSecretFile(t, tempGitlabShellDir, secretToken)
 
-	config.Config.Gitlab.URL = ts.URL
+	config.Config.Gitlab.URL = serverURL
 	config.Config.Gitlab.SecretFile = filepath.Join(tempGitlabShellDir, ".gitlab_shell_secret")
 	config.Config.Gitlab.HTTPSettings.User = gitlabUser
 	config.Config.Gitlab.HTTPSettings.Password = gitlabPassword
@@ -138,21 +227,18 @@ func testHooksPrePostReceive(t *testing.T) {
 
 	hookNames := []string{"pre-receive", "post-receive"}
 
-	featureSets, err := testhelper.NewFeatureSets([]featureflag.FeatureFlag{featureflag.GoPreReceiveHook, featureflag.GoPostReceiveHook})
-	require.NoError(t, err)
-
 	for _, hookName := range hookNames {
-		for _, featureSet := range featureSets {
-			t.Run(fmt.Sprintf("hookName: %s, disabled feature flags: %s", hookName, featureSet), func(t *testing.T) {
+		for _, useGoPostReceive := range []bool{true, false} {
+			t.Run(fmt.Sprintf("hookName: %s, using Go PostReceive: %v", hookName, useGoPostReceive), func(t *testing.T) {
 				customHookOutputPath, cleanup := testhelper.WriteEnvToCustomHook(t, testRepoPath, hookName)
 				defer cleanup()
 
-				config.Config.Gitlab.URL = ts.URL
+				config.Config.Gitlab.URL = serverURL
 				config.Config.Gitlab.SecretFile = filepath.Join(tempGitlabShellDir, ".gitlab_shell_secret")
 				config.Config.Gitlab.HTTPSettings.User = gitlabUser
 				config.Config.Gitlab.HTTPSettings.Password = gitlabPassword
 
-				gitlabAPI, err := hook.NewGitlabAPI(config.Config.Gitlab)
+				gitlabAPI, err := gitalyhook.NewGitlabAPI(config.Config.Gitlab)
 				require.NoError(t, err)
 
 				socket, stop := runHookServiceServerWithAPI(t, token, gitlabAPI)
@@ -166,7 +252,7 @@ func testHooksPrePostReceive(t *testing.T) {
 				cmd.Stderr = &stderr
 				cmd.Stdout = &stdout
 				cmd.Stdin = stdin
-				cmd.Env = testhelper.EnvForHooks(
+				cmd.Env = append(testhelper.EnvForHooks(
 					t,
 					tempGitlabShellDir,
 					socket,
@@ -186,11 +272,7 @@ func testHooksPrePostReceive(t *testing.T) {
 						NoProxy:    noProxy,
 					},
 					gitPushOptions...,
-				)
-
-				if !featureSet.IsDisabled(featureflag.GoPreReceiveHook) {
-					cmd.Env = append(cmd.Env, fmt.Sprintf("%s=true", featureflag.GoPreReceiveHookEnvVar))
-				}
+				), fmt.Sprintf("%s=%v", featureflag.GoPostReceiveHookEnvVar, useGoPostReceive))
 
 				cmd.Dir = testRepoPath
 
@@ -253,24 +335,17 @@ func TestHooksUpdate(t *testing.T) {
 	socket, stop := runHookServiceServer(t, token)
 	defer stop()
 
-	featureSets, err := testhelper.NewFeatureSets([]featureflag.FeatureFlag{featureflag.GoUpdateHook})
-	require.NoError(t, err)
+	config.Config.Hooks.CustomHooksDir = customHooksDir
 
-	for _, featureSet := range featureSets {
-		t.Run(fmt.Sprintf("disabled %v", featureSet), func(t *testing.T) {
-			config.Config.Hooks.CustomHooksDir = customHooksDir
-
-			testHooksUpdate(t, tempGitlabShellDir, socket, token, testhelper.GlHookValues{
-				GLID:       glID,
-				GLUsername: glUsername,
-				GLRepo:     glRepository,
-				GLProtocol: glProtocol,
-			}, featureSet)
-		})
-	}
+	testHooksUpdate(t, tempGitlabShellDir, socket, token, testhelper.GlHookValues{
+		GLID:       glID,
+		GLUsername: glUsername,
+		GLRepo:     glRepository,
+		GLProtocol: glProtocol,
+	})
 }
 
-func testHooksUpdate(t *testing.T, gitlabShellDir, socket, token string, glValues testhelper.GlHookValues, features testhelper.FeatureSet) {
+func testHooksUpdate(t *testing.T, gitlabShellDir, socket, token string, glValues testhelper.GlHookValues) {
 	defer func(cfg config.Cfg) {
 		config.Config = cfg
 	}(config.Config)
@@ -304,9 +379,6 @@ open('%s', 'w') { |f| f.puts(JSON.dump(ARGV)) }
 
 	var stdout, stderr bytes.Buffer
 
-	if !features.IsDisabled(featureflag.GoUpdateHook) {
-		cmd.Env = append(cmd.Env, fmt.Sprintf("%s=true", featureflag.GoUpdateHookEnvVar))
-	}
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	cmd.Dir = testRepoPath
@@ -362,18 +434,18 @@ func TestHooksPostReceiveFailed(t *testing.T) {
 		PostReceiveCounterDecreased: false,
 		Protocol:                    "ssh",
 	}
-	ts := testhelper.NewGitlabTestServer(c)
-	defer ts.Close()
+	serverURL, cleanup := testhelper.NewGitlabTestServer(t, c)
+	defer cleanup()
 
 	testhelper.WriteShellSecretFile(t, tempGitlabShellDir, secretToken)
 
 	config.Config.GitlabShell.Dir = tempGitlabShellDir
-	config.Config.Gitlab.URL = ts.URL
+	config.Config.Gitlab.URL = serverURL
 	config.Config.Gitlab.SecretFile = filepath.Join(tempGitlabShellDir, ".gitlab_shell_secret")
 
 	token := "abc123"
 
-	gitlabAPI, err := hook.NewGitlabAPI(config.Config.Gitlab)
+	gitlabAPI, err := gitalyhook.NewGitlabAPI(config.Config.Gitlab)
 	require.NoError(t, err)
 
 	socket, stop := runHookServiceServerWithAPI(t, token, gitlabAPI)
@@ -484,14 +556,14 @@ func TestHooksNotAllowed(t *testing.T) {
 		PostReceiveCounterDecreased: true,
 		Protocol:                    "ssh",
 	}
-	ts := testhelper.NewGitlabTestServer(c)
-	defer ts.Close()
+	serverURL, cleanup := testhelper.NewGitlabTestServer(t, c)
+	defer cleanup()
 
-	testhelper.WriteTemporaryGitlabShellConfigFile(t, tempGitlabShellDir, testhelper.GitlabShellConfig{GitlabURL: ts.URL})
+	testhelper.WriteTemporaryGitlabShellConfigFile(t, tempGitlabShellDir, testhelper.GitlabShellConfig{GitlabURL: serverURL})
 	testhelper.WriteShellSecretFile(t, tempGitlabShellDir, "the wrong token")
 
 	config.Config.GitlabShell.Dir = tempGitlabShellDir
-	config.Config.Gitlab.URL = ts.URL
+	config.Config.Gitlab.URL = serverURL
 	config.Config.Gitlab.SecretFile = filepath.Join(tempGitlabShellDir, ".gitlab_shell_secret")
 
 	customHookOutputPath, cleanup := testhelper.WriteEnvToCustomHook(t, testRepoPath, "post-receive")
@@ -499,7 +571,7 @@ func TestHooksNotAllowed(t *testing.T) {
 
 	token := "abc123"
 
-	gitlabAPI, err := hook.NewGitlabAPI(config.Config.Gitlab)
+	gitlabAPI, err := gitalyhook.NewGitlabAPI(config.Config.Gitlab)
 	require.NoError(t, err)
 
 	socket, stop := runHookServiceServerWithAPI(t, token, gitlabAPI)
@@ -543,8 +615,8 @@ func TestCheckOK(t *testing.T) {
 		PostReceiveCounterDecreased: false,
 		Protocol:                    "ssh",
 	}
-	ts := testhelper.NewGitlabTestServer(c)
-	defer ts.Close()
+	serverURL, cleanup := testhelper.NewGitlabTestServer(t, c)
+	defer cleanup()
 
 	tempDir, err := ioutil.TempDir("", t.Name())
 	require.NoError(t, err)
@@ -563,9 +635,9 @@ func TestCheckOK(t *testing.T) {
 	testhelper.WriteShellSecretFile(t, gitlabShellDir, "the secret")
 	testhelper.WriteTemporaryGitlabShellConfigFile(t,
 		gitlabShellDir,
-		testhelper.GitlabShellConfig{GitlabURL: ts.URL, HTTPSettings: testhelper.HTTPSettings{User: user, Password: password}})
+		testhelper.GitlabShellConfig{GitlabURL: serverURL, HTTPSettings: testhelper.HTTPSettings{User: user, Password: password}})
 
-	configPath, cleanup := testhelper.WriteTemporaryGitalyConfigFile(t, tempDir, ts.URL, user, password)
+	configPath, cleanup := testhelper.WriteTemporaryGitalyConfigFile(t, tempDir, serverURL, user, password)
 	defer cleanup()
 	cmd := exec.Command(fmt.Sprintf("%s/gitaly-hooks", config.Config.BinDir), "check", configPath)
 
@@ -592,8 +664,8 @@ func TestCheckBadCreds(t *testing.T) {
 		Protocol:                    "ssh",
 		GitPushOptions:              nil,
 	}
-	ts := testhelper.NewGitlabTestServer(c)
-	defer ts.Close()
+	serverURL, cleanup := testhelper.NewGitlabTestServer(t, c)
+	defer cleanup()
 
 	tempDir, err := ioutil.TempDir("", t.Name())
 	require.NoError(t, err)
@@ -609,10 +681,10 @@ func TestCheckBadCreds(t *testing.T) {
 	require.NoError(t, err)
 	require.NoError(t, os.Symlink(filepath.Join(cwd, "../../ruby/gitlab-shell/bin/check"), filepath.Join(binDir, "check")))
 
-	testhelper.WriteTemporaryGitlabShellConfigFile(t, gitlabShellDir, testhelper.GitlabShellConfig{GitlabURL: ts.URL, HTTPSettings: testhelper.HTTPSettings{User: user + "wrong", Password: password}})
+	testhelper.WriteTemporaryGitlabShellConfigFile(t, gitlabShellDir, testhelper.GitlabShellConfig{GitlabURL: serverURL, HTTPSettings: testhelper.HTTPSettings{User: user + "wrong", Password: password}})
 	testhelper.WriteShellSecretFile(t, gitlabShellDir, "the secret")
 
-	configPath, cleanup := testhelper.WriteTemporaryGitalyConfigFile(t, tempDir, ts.URL, "wrong", password)
+	configPath, cleanup := testhelper.WriteTemporaryGitalyConfigFile(t, tempDir, serverURL, "wrong", password)
 	defer cleanup()
 
 	cmd := exec.Command(fmt.Sprintf("%s/gitaly-hooks", config.Config.BinDir), "check", configPath)
@@ -627,13 +699,13 @@ func TestCheckBadCreds(t *testing.T) {
 }
 
 func runHookServiceServer(t *testing.T, token string) (string, func()) {
-	return runHookServiceServerWithAPI(t, token, hook.GitlabAPIStub)
+	return runHookServiceServerWithAPI(t, token, gitalyhook.GitlabAPIStub)
 }
 
-func runHookServiceServerWithAPI(t *testing.T, token string, gitlabAPI hook.GitlabAPI) (string, func()) {
+func runHookServiceServerWithAPI(t *testing.T, token string, gitlabAPI gitalyhook.GitlabAPI) (string, func()) {
 	server := testhelper.NewServerWithAuth(t, nil, nil, token)
 
-	gitalypb.RegisterHookServiceServer(server.GrpcServer(), hook.NewServer(gitlabAPI, config.Config.Hooks))
+	gitalypb.RegisterHookServiceServer(server.GrpcServer(), hook.NewServer(gitalyhook.NewManager(gitlabAPI, config.Config)))
 	reflection.Register(server.GrpcServer())
 	require.NoError(t, server.Start())
 
