@@ -1,19 +1,22 @@
 package main
 
 import (
+	"context"
 	"flag"
 	"fmt"
 	"os"
 	"time"
 
+	"github.com/prometheus/client_golang/prometheus"
 	log "github.com/sirupsen/logrus"
 	"gitlab.com/gitlab-org/gitaly/internal/bootstrap"
 	"gitlab.com/gitlab-org/gitaly/internal/bootstrap/starter"
-	"gitlab.com/gitlab-org/gitaly/internal/config"
-	"gitlab.com/gitlab-org/gitaly/internal/config/sentry"
 	"gitlab.com/gitlab-org/gitaly/internal/git"
-	"gitlab.com/gitlab-org/gitaly/internal/server"
-	"gitlab.com/gitlab-org/gitaly/internal/service/hook"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config/sentry"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/hook"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/server"
+	glog "gitlab.com/gitlab-org/gitaly/internal/log"
 	"gitlab.com/gitlab-org/gitaly/internal/storage"
 	"gitlab.com/gitlab-org/gitaly/internal/tempdir"
 	"gitlab.com/gitlab-org/gitaly/internal/version"
@@ -99,7 +102,10 @@ func run(b *bootstrap.Bootstrap) error {
 		}
 	}
 
-	servers := server.NewGitalyServerFactory(gitlabAPI)
+	hookManager := hook.NewManager(gitlabAPI, config.Config)
+	prometheus.MustRegister(hookManager)
+
+	servers := server.NewGitalyServerFactory(hookManager)
 	defer servers.Stop()
 
 	b.StopAction = servers.GracefulStop
@@ -163,5 +169,12 @@ func run(b *bootstrap.Bootstrap) error {
 		return fmt.Errorf("initialize gitaly-ruby: %v", err)
 	}
 
+	ctx := context.Background()
+	shutdownWorkers, err := servers.StartWorkers(ctx, glog.Default(), config.Config)
+	if err != nil {
+		return fmt.Errorf("initialize auxiliary workers: %v", err)
+	}
+	defer shutdownWorkers()
+
 	return b.Wait(config.Config.GracefulRestartTimeout.Duration())
 }
