@@ -2,7 +2,7 @@ package alternates
 
 import (
 	"fmt"
-	"path"
+	"path/filepath"
 	"strings"
 
 	"gitlab.com/gitlab-org/gitaly/internal/git/repository"
@@ -20,14 +20,14 @@ func PathAndEnv(repo repository.GitRepo) (string, []string, error) {
 
 	var env []string
 	if dir := repo.GetGitObjectDirectory(); dir != "" {
-		env = append(env, fmt.Sprintf("GIT_OBJECT_DIRECTORY=%s", path.Join(repoPath, dir)))
+		env = append(env, fmt.Sprintf("GIT_OBJECT_DIRECTORY=%s", filepath.Join(repoPath, dir)))
 	}
 
 	if dirs := repo.GetGitAlternateObjectDirectories(); len(dirs) > 0 {
 		var dirsList []string
 
 		for _, dir := range dirs {
-			dirsList = append(dirsList, path.Join(repoPath, dir))
+			dirsList = append(dirsList, filepath.Join(repoPath, dir))
 		}
 
 		env = append(env, fmt.Sprintf("GIT_ALTERNATE_OBJECT_DIRECTORIES=%s", strings.Join(dirsList, ":")))
