@@ -9,16 +9,12 @@ type FeatureFlag struct {
 // In order to support coverage of combined features usage all feature flags should be marked as enabled for the test.
 // NOTE: if you add a new feature flag please add it to the `All` list defined below.
 var (
-	// GoUpdateHook will bypass the ruby update hook and use the go implementation of custom hooks
-	GoUpdateHook = FeatureFlag{Name: "go_update_hook", OnByDefault: true}
 	// GoFetchSourceBranch enables a go implementation of FetchSourceBranch
 	GoFetchSourceBranch = FeatureFlag{Name: "go_fetch_source_branch", OnByDefault: false}
 	// DistributedReads allows praefect to redirect accessor operations to up-to-date secondaries
 	DistributedReads = FeatureFlag{Name: "distributed_reads", OnByDefault: true}
-	// GoPreReceiveHook will bypass the ruby pre-receive hook and use the go implementation
-	GoPreReceiveHook = FeatureFlag{Name: "go_prereceive_hook", OnByDefault: true}
 	// GoPostReceiveHook will bypass the ruby post-receive hook and use the go implementation
-	GoPostReceiveHook = FeatureFlag{Name: "go_postreceive_hook", OnByDefault: false}
+	GoPostReceiveHook = FeatureFlag{Name: "go_postreceive_hook", OnByDefault: true}
 	// ReferenceTransactions will handle Git reference updates via the transaction service for strong consistency
 	ReferenceTransactions = FeatureFlag{Name: "reference_transactions", OnByDefault: true}
 	// ReferenceTransactionsOperationService will enable reference transactions for the OperationService
@@ -29,20 +25,23 @@ var (
 	ReferenceTransactionsSSHService = FeatureFlag{Name: "reference_transactions_ssh_service", OnByDefault: true}
 	// ReferenceTranasctiionsPrimaryWins will change transaction registration such that
 	// secondaries will take part in transactions, but not influence their outcome.
-	ReferenceTransactionsPrimaryWins = FeatureFlag{Name: "reference_transactions_primary_wins", OnByDefault: true}
+	ReferenceTransactionsPrimaryWins = FeatureFlag{Name: "reference_transactions_primary_wins", OnByDefault: false}
 	// ReferenceTransactionHook will enable the reference-transaction hook
 	// introduced with Git v2.28.0 for voting on transactions
-	ReferenceTransactionHook = FeatureFlag{Name: "reference_transaction_hook", OnByDefault: false}
+	ReferenceTransactionHook = FeatureFlag{Name: "reference_transaction_hook", OnByDefault: true}
+	// RubyReferenceTransactionHook will enable the reference-transaction hook
+	// introduced with Git v2.28.0 for voting on transactions in the Ruby sidecar.
+	RubyReferenceTransactionHook = FeatureFlag{Name: "ruby_reference_transaction_hook", OnByDefault: false}
 	// LogCommandStats will log additional rusage stats for commands
 	LogCommandStats = FeatureFlag{Name: "log_command_stats", OnByDefault: false}
+	// GoUserMergeBranch enables the Go implementation of UserMergeBranch
+	GoUserMergeBranch = FeatureFlag{Name: "go_user_merge_branch", OnByDefault: false}
 )
 
 // All includes all feature flags.
 var All = []FeatureFlag{
-	GoUpdateHook,
 	GoFetchSourceBranch,
 	DistributedReads,
-	GoPreReceiveHook,
 	GoPostReceiveHook,
 	ReferenceTransactions,
 	ReferenceTransactionsOperationService,
@@ -50,11 +49,11 @@ var All = []FeatureFlag{
 	ReferenceTransactionsSSHService,
 	ReferenceTransactionsPrimaryWins,
 	ReferenceTransactionHook,
+	RubyReferenceTransactionHook,
+	GoUserMergeBranch,
 }
 
 const (
-	GoUpdateHookEnvVar             = "GITALY_GO_UPDATE"
-	GoPreReceiveHookEnvVar         = "GITALY_GO_PRERECEIVE"
 	GoPostReceiveHookEnvVar        = "GITALY_GO_POSTRECEIVE"
 	ReferenceTransactionHookEnvVar = "GITALY_REFERENCE_TRANSACTION_HOOK"
 )
