@@ -75,7 +75,7 @@ func TestStreamDirectorReadOnlyEnforcement(t *testing.T) {
 			ctx, cancel := testhelper.Context()
 			defer cancel()
 
-			rs := datastore.NewMemoryRepositoryStore(nil)
+			rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 			require.NoError(t, rs.SetGeneration(ctx, virtualStorage, relativePath, storage, 1))
 			if tc.readOnly {
 				require.NoError(t, rs.SetGeneration(ctx, virtualStorage, relativePath, storage, 0))
@@ -84,14 +84,14 @@ func TestStreamDirectorReadOnlyEnforcement(t *testing.T) {
 			coordinator := NewCoordinator(
 				datastore.NewMemoryReplicationEventQueue(conf),
 				rs,
-				&nodes.MockManager{GetShardFunc: func(vs string) (nodes.Shard, error) {
+				NewNodeManagerRouter(&nodes.MockManager{GetShardFunc: func(vs string) (nodes.Shard, error) {
 					require.Equal(t, virtualStorage, vs)
 					return nodes.Shard{
 						Primary: &nodes.MockNode{GetStorageMethod: func() string {
 							return storage
 						}},
 					}, nil
-				}},
+				}}, rs),
 				transactions.NewManager(conf),
 				conf,
 				protoregistry.GitalyProtoPreregistered,
@@ -151,16 +151,17 @@ func TestStreamDirectorMutator(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
 	txMgr := transactions.NewManager(conf)
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
 	coordinator := NewCoordinator(
 		queueInterceptor,
-		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
-		nodeMgr,
+		rs,
+		NewNodeManagerRouter(nodeMgr, rs),
 		txMgr,
 		conf,
 		protoregistry.GitalyProtoPreregistered,
@@ -247,15 +248,29 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 			},
 		},
 		{
-			// Currently, transactions are created such that all nodes need to agree.
-			// This is going to change in the future, but for now let's just test that
-			// we don't get any replication jobs if any node disagrees.
 			desc:            "failing vote should not create replication jobs",
 			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
 			nodes: []node{
-				{primary: true, vote: "foobar", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
-				{primary: false, vote: "foobar", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
-				{primary: false, vote: "barfoo", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+				{primary: true, vote: "foo", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "qux", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "bar", shouldSucceed: false, shouldGetRepl: false, shouldParticipate: true},
+			},
+		},
+		{
+			desc:            "primary should reach quorum with disagreeing secondary",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
+			nodes: []node{
+				{primary: true, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "barfoo", shouldSucceed: false, shouldGetRepl: true, shouldParticipate: true},
+			},
+		},
+		{
+			desc:            "quorum should create replication jobs for disagreeing node",
+			disableFeatures: []featureflag.FeatureFlag{featureflag.ReferenceTransactionsPrimaryWins},
+			nodes: []node{
+				{primary: true, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "foobar", shouldSucceed: true, shouldGetRepl: false, shouldParticipate: true},
+				{primary: false, vote: "barfoo", shouldSucceed: false, shouldGetRepl: true, shouldParticipate: true},
 			},
 		},
 		{
@@ -344,7 +359,7 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 				ctx = featureflag.IncomingCtxWithDisabledFeatureFlag(ctx, feature)
 			}
 
-			nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+			nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 			require.NoError(t, err)
 			nodeMgr.Start(0, time.Hour)
 
@@ -369,7 +384,14 @@ func TestStreamDirectorMutator_Transaction(t *testing.T) {
 				require.NoError(t, rs.SetGeneration(ctx, repo.StorageName, repo.RelativePath, storageNodes[i].Storage, n.generation))
 			}
 
-			coordinator := NewCoordinator(queueInterceptor, rs, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+			coordinator := NewCoordinator(
+				queueInterceptor,
+				rs,
+				NewNodeManagerRouter(nodeMgr, rs),
+				txMgr,
+				conf,
+				protoregistry.GitalyProtoPreregistered,
+			)
 
 			fullMethod := "/gitaly.SmartHTTPService/PostReceivePack"
 
@@ -490,16 +512,17 @@ func TestStreamDirectorAccessor(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Minute)
 
 	txMgr := transactions.NewManager(conf)
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
 	coordinator := NewCoordinator(
 		queue,
-		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
-		nodeMgr,
+		rs,
+		NewNodeManagerRouter(nodeMgr, rs),
 		txMgr,
 		conf,
 		protoregistry.GitalyProtoPreregistered,
@@ -579,7 +602,7 @@ func TestCoordinatorStreamDirector_distributesReads(t *testing.T) {
 	require.NoError(t, err)
 	require.NoError(t, repoStore.SetGeneration(ctx, conf.VirtualStorages[0].Name, targetRepo.RelativePath, secondaryNodeConf.Storage, generation))
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, repoStore, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, repoStore, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Minute)
 
@@ -588,7 +611,7 @@ func TestCoordinatorStreamDirector_distributesReads(t *testing.T) {
 	coordinator := NewCoordinator(
 		queue,
 		repoStore,
-		nodeMgr,
+		NewNodeManagerRouter(nodeMgr, repoStore),
 		txMgr,
 		conf,
 		protoregistry.GitalyProtoPreregistered,
@@ -778,16 +801,17 @@ func TestAbsentCorrelationID(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
 	txMgr := transactions.NewManager(conf)
+	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
 	coordinator := NewCoordinator(
 		queueInterceptor,
-		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
-		nodeMgr,
+		rs,
+		NewNodeManagerRouter(nodeMgr, rs),
 		txMgr,
 		conf,
 		protoregistry.GitalyProtoPreregistered,
@@ -884,104 +908,6 @@ func TestCoordinatorEnqueueFailure(t *testing.T) {
 	require.Equal(t, err.Error(), "rpc error: code = Unknown desc = enqueue replication event: "+expectErrMsg)
 }
 
-func TestStreamDirectorServerScope(t *testing.T) {
-	gz := proto.FileDescriptor("mock.proto")
-	fd, err := protoregistry.ExtractFileDescriptor(gz)
-	require.NoError(t, err)
-
-	registry, err := protoregistry.New(fd)
-	require.NoError(t, err)
-
-	conf := config.Config{VirtualStorages: []*config.VirtualStorage{{Name: "praefect"}, {Name: "another"}}}
-	mgr := &nodes.MockManager{GetShardFunc: func(s string) (nodes.Shard, error) {
-		require.Equal(t, "praefect", s)
-		return nodes.Shard{Primary: &nodes.MockNode{}}, nil
-	}}
-	coordinator := NewCoordinator(
-		nil,
-		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
-		mgr,
-		nil,
-		conf,
-		registry,
-	)
-
-	fullMethod := "/mock.SimpleService/ServerAccessor"
-	requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
-
-	frame, err := proto.Marshal(&mock.SimpleRequest{})
-	require.NoError(t, err)
-
-	ctx, cancel := testhelper.Context()
-	defer cancel()
-
-	sp, err := coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
-	require.NoError(t, err)
-	require.NotNil(t, sp.Primary())
-	require.Empty(t, sp.Secondaries())
-}
-
-func TestStreamDirectorServerScope_Error(t *testing.T) {
-	gz := proto.FileDescriptor("mock.proto")
-	fd, err := protoregistry.ExtractFileDescriptor(gz)
-	require.NoError(t, err)
-
-	registry, err := protoregistry.New(fd)
-	require.NoError(t, err)
-
-	conf := config.Config{VirtualStorages: []*config.VirtualStorage{{Name: "fake"}, {Name: "another"}}}
-	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
-
-	t.Run("unknown storage provided", func(t *testing.T) {
-		mgr := &nodes.MockManager{
-			GetShardFunc: func(s string) (nodes.Shard, error) {
-				require.Equal(t, "fake", s)
-				return nodes.Shard{}, nodes.ErrVirtualStorageNotExist
-			},
-		}
-		coordinator := NewCoordinator(nil, rs, mgr, nil, conf, registry)
-
-		fullMethod := "/mock.SimpleService/ServerAccessor"
-		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
-
-		frame, err := proto.Marshal(&mock.SimpleRequest{})
-		require.NoError(t, err)
-
-		ctx, cancel := testhelper.Context()
-		defer cancel()
-
-		_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
-		require.Error(t, err)
-		result, ok := status.FromError(err)
-		require.True(t, ok)
-		require.Equal(t, codes.InvalidArgument, result.Code())
-		require.Equal(t, "virtual storage does not exist", result.Message())
-	})
-
-	t.Run("primary not healthy", func(t *testing.T) {
-		mgr := &nodes.MockManager{
-			GetShardFunc: func(s string) (nodes.Shard, error) {
-				require.Equal(t, "fake", s)
-				return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
-			},
-		}
-		coordinator := NewCoordinator(nil, rs, mgr, nil, conf, registry)
-
-		fullMethod := "/mock.SimpleService/ServerAccessor"
-		requireScopeOperation(t, coordinator.registry, fullMethod, protoregistry.ScopeServer, protoregistry.OpAccessor)
-
-		frame, err := proto.Marshal(&mock.SimpleRequest{})
-		require.NoError(t, err)
-
-		ctx, cancel := testhelper.Context()
-		defer cancel()
-
-		_, err = coordinator.StreamDirector(ctx, fullMethod, &mockPeeker{frame})
-		require.Error(t, err)
-		require.Equal(t, nodes.ErrPrimaryNotHealthy, err)
-	})
-}
-
 func TestStreamDirectorStorageScope(t *testing.T) {
 	// stubs health-check requests because nodes.NewManager establishes connection on creation
 	gitalySocket0, gitalySocket1 := testhelper.GetTemporaryGitalySocketFileName(), testhelper.GetTemporaryGitalySocketFileName()
@@ -1002,10 +928,17 @@ func TestStreamDirectorStorageScope(t *testing.T) {
 
 	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-	nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Second)
-	coordinator := NewCoordinator(nil, rs, nodeMgr, nil, conf, protoregistry.GitalyProtoPreregistered)
+	coordinator := NewCoordinator(
+		nil,
+		rs,
+		NewNodeManagerRouter(nodeMgr, rs),
+		nil,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	ctx, cancel := testhelper.Context()
 	defer cancel()
@@ -1062,10 +995,12 @@ func TestStreamDirectorStorageScopeError(t *testing.T) {
 				return nodes.Shard{}, assert.AnError
 			},
 		}
+
+		rs := datastore.NewMemoryRepositoryStore(nil)
 		coordinator := NewCoordinator(
 			nil,
-			datastore.NewMemoryRepositoryStore(nil),
-			mgr,
+			rs,
+			NewNodeManagerRouter(mgr, rs),
 			nil,
 			config.Config{},
 			protoregistry.GitalyProtoPreregistered,
@@ -1089,10 +1024,12 @@ func TestStreamDirectorStorageScopeError(t *testing.T) {
 				return nodes.Shard{}, nodes.ErrVirtualStorageNotExist
 			},
 		}
+
+		rs := datastore.NewMemoryRepositoryStore(nil)
 		coordinator := NewCoordinator(
 			nil,
-			datastore.NewMemoryRepositoryStore(nil),
-			mgr,
+			rs,
+			NewNodeManagerRouter(mgr, rs),
 			nil,
 			config.Config{},
 			protoregistry.GitalyProtoPreregistered,
@@ -1117,10 +1054,12 @@ func TestStreamDirectorStorageScopeError(t *testing.T) {
 					return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
 				},
 			}
+
+			rs := datastore.NewMemoryRepositoryStore(nil)
 			coordinator := NewCoordinator(
 				nil,
-				datastore.NewMemoryRepositoryStore(nil),
-				mgr,
+				rs,
+				NewNodeManagerRouter(mgr, rs),
 				nil,
 				config.Config{},
 				protoregistry.GitalyProtoPreregistered,
@@ -1137,7 +1076,7 @@ func TestStreamDirectorStorageScopeError(t *testing.T) {
 			result, ok := status.FromError(err)
 			require.True(t, ok)
 			require.Equal(t, codes.Internal, result.Code())
-			require.Equal(t, `accessor storage scoped: get shard "fake": primary is not healthy`, result.Message())
+			require.Equal(t, `accessor storage scoped: route storage accessor "fake": primary is not healthy`, result.Message())
 		})
 
 		t.Run("mutator", func(t *testing.T) {
@@ -1147,10 +1086,11 @@ func TestStreamDirectorStorageScopeError(t *testing.T) {
 					return nodes.Shard{}, nodes.ErrPrimaryNotHealthy
 				},
 			}
+			rs := datastore.NewMemoryRepositoryStore(nil)
 			coordinator := NewCoordinator(
 				nil,
-				datastore.NewMemoryRepositoryStore(nil),
-				mgr,
+				rs,
+				NewNodeManagerRouter(mgr, rs),
 				nil,
 				config.Config{},
 				protoregistry.GitalyProtoPreregistered,
