@@ -26,10 +26,6 @@ type ReplicationEventQueue interface {
 	// 'completed'. Otherwise it won't be changed.
 	// It returns sub-set of passed in ids that were updated.
 	Acknowledge(ctx context.Context, state JobState, ids []uint64) ([]uint64, error)
-	// GetOutdatedRepositories returns storages by repositories which are considered outdated. A repository is considered
-	// outdated if the latest replication job is not in 'complete' state or the latest replication job does not originate
-	// from the reference storage.
-	GetOutdatedRepositories(ctx context.Context, virtualStorage string, referenceStorage string) (map[string][]string, error)
 	// StartHealthUpdate starts periodical update of the event's health identifier.
 	// The events with fresh health identifier won't be considered as stale.
 	// The health update will be executed on each new entry received from trigger channel passed in.
@@ -165,10 +161,49 @@ func NewPostgresReplicationEventQueue(qc glsql.Querier) PostgresReplicationEvent
 
 // PostgresReplicationEventQueue is a Postgres implementation of persistent queue.
 type PostgresReplicationEventQueue struct {
+	// The main requirements for the queue implementation are:
+	//  - it should not use long transactions
+	//  - it should perform without problems if PgBouncer is used in between with `pool_mode = transaction`
+	//  - it should perform concurrently with other queue implementations (support of horizontal scaling)
+	//  - it should handle events sequentially starting with the oldest
+	//  - it should handle events concurrently for multiple repositories
+	//  - it should support retries
+	//
+	// Current implementation uses the following tables to mimic the queue:
+	//  - replication_queue_lock
+	//  - replication_queue
+	//  - replication_queue_job_lock
+	//
+	// `replication_queue_lock` holds repository level locks to synchronize multiple Praefects instances
+	// working on the same queue (shared database). Only one worker is allowed to operate on a given repository at a time.
+	// The `id` column is a concatenated string of the virtual storage name, gitaly node,
+	// and repository relative path: virtual1|node1|/path/to/project.git with `|` as a delimiter (represented as <lock>
+	// elsewhere in this doc).
+	// The `acquired` column reflects whether the lock for this repository (qualified by `id` column) is taken.
+	//
+	// `replication_queue` stores the actual replication event. The job is stored as a JSONB value in the `job` column of
+	// the table. It includes `meta` column designed to store meta information such as `correlation_id` etc. Each event has
+	// corresponding value in the `lock_id` column from `replication_queue_lock` table. Each replication event will be
+	// created with the following defaults:
+	//  - attempt: 3
+	//  - state: `ready`
+	//  - created_at: UTC timestamp
+	//  - updated_at: NULL
+	//
+	// `replication_queue_job_lock` holds event specific locks to prevent multiple queue workers from operating on the same
+	// event and track the events that are protected by the <lock>.
+	//
+	// The mechanics of how the queue works is described in the `Enqueue`, `Dequeue`, `Acknowledge` methods.
 	qc glsql.Querier
 }
 
 func (rq PostgresReplicationEventQueue) Enqueue(ctx context.Context, event ReplicationEvent) (ReplicationEvent, error) {
+	// When `Enqueue` method is called:
+	//  1. Insertion of the new record into `replication_queue_lock` table, so we are ensured all events have
+	//     a corresponding <lock>. If a record already exists it won't be inserted again.
+	//  2. Insertion of the new record into the `replication_queue` table with the defaults listed above,
+	//     the job, the meta and corresponding <lock> used in `replication_queue_lock` table for the `lock_id` column.
+
 	query := `
 		WITH insert_lock AS (
 			INSERT INTO replication_queue_lock(id)
@@ -195,6 +230,23 @@ func (rq PostgresReplicationEventQueue) Enqueue(ctx context.Context, event Repli
 }
 
 func (rq PostgresReplicationEventQueue) Dequeue(ctx context.Context, virtualStorage, nodeStorage string, count int) ([]ReplicationEvent, error) {
+	// When `Dequeue` method is called:
+	//  1. Events with attempts left that are either in `ready` or `failed` state are candidates for dequeuing.
+	//     Events already being processed by another worker are filtered out by checking if the event is already locked
+	//     in the `replication_queue_job_lock` table.
+	//  2. Events for repositories that are already locked by another Praefect instance are filtered out.
+	//     Repository locks are stored in the `replication_queue_lock` table.
+	//  3. The events that still remain after filtering are dequeued. On dequeuing:
+	//      - The event's attempts are decremented by 1.
+	//      - The event's state is set to `in_progress`
+	//      - The event's `updated_at` is set to current time in UTC.
+	//  4. For each event retrieved from the step above a new record would be created in
+	//     `replication_queue_job_lock` table. Rows in this table allows us to track events that were fetched for processing
+	//     and relation of them with the locks in the `replication_queue_lock` table. The reason we need it is because
+	//     multiple events can be fetched for the same repository (more details on it in `Acknowledge` below).
+	//  5. Update the corresponding <lock> in `replication_queue_lock` table and column `acquired` is assigned with
+	//     `TRUE` value to signal that this <lock> is busy and can't be used to fetch events (step 2.).
+
 	query := `
 		WITH lock AS (
 			SELECT id
@@ -254,6 +306,25 @@ func (rq PostgresReplicationEventQueue) Dequeue(ctx context.Context, virtualStor
 }
 
 func (rq PostgresReplicationEventQueue) Acknowledge(ctx context.Context, state JobState, ids []uint64) ([]uint64, error) {
+	// When `Acknowledge` method is called:
+	//  1. The list of event `id`s and corresponding <lock>s retrieved from `replication_queue` table as passed in by the
+	//     user `ids` could not exist in the table or the `state` of the event could differ from `in_progress` (it is
+	//     possible to acknowledge only events previously fetched by the `Dequeue` method)
+	//  2. Based on the list fetched on previous step the delete is executed on the `replication_queue` table. In case the
+	//     new state for the entry is 'dead' it will be just deleted, but if the new state is 'completed' the event will
+	//     be delete as well, but all events similar to it (events for the same repository with same change type and a source)
+	//     that were created before processed events were queued for processing will also be deleted.
+	//     In case the new state is something different ('failed') the event will be updated only with a new state.
+	//     It returns a list of event `id`s and corresponding <lock>s of the affected events during this delete/update process.
+	//  3. The removal of records in `replication_queue_job_lock` table happens that were created by step 4. of `Dequeue`
+	//     method call.
+	//  4. Acquisition state of <lock>s in `replication_queue_lock` table updated by comparing amount of existing bindings
+	//     in `replication_queue_lock` table for the <lock> to amount of removed bindings done on the 3. for the <lock>
+	//     and if amount is the same the <lock> is free and column `acquired` assigned `FALSE` value, so this <lock> can
+	//     be used in the `Dequeue` method to retrieve other events. If amounts don't match no update happens and <lock>
+	//     remains acquired until all events are acknowledged (binding records removed from the `replication_queue_job_lock`
+	//     table).
+
 	if len(ids) == 0 {
 		return nil, nil
 	}
@@ -275,33 +346,39 @@ func (rq PostgresReplicationEventQueue) Acknowledge(ctx context.Context, state J
 			AND state = 'in_progress'
 			FOR UPDATE
 		)
-		, to_release AS (
+		, deleted AS (
+			DELETE FROM replication_queue AS queue
+			USING existing
+			WHERE ($2::REPLICATION_JOB_STATE = 'dead' AND existing.id = queue.id) OR (
+				$2::REPLICATION_JOB_STATE = 'completed'
+				AND (existing.id = queue.id OR (
+					-- this is an optimization to omit events that won't make any effect as the same event
+					-- was just applied, so we acknowledge similar events:
+					-- only not yet touched events (no attempts to process it)
+					queue.state = 'ready'
+					-- and they were created before current event was consumed for processing
+					AND queue.created_at < existing.updated_at
+					-- they are for the exact same repository
+					AND queue.lock_id = existing.lock_id
+					-- and created to apply exact same replication operation (gc, update, ...)
+					AND queue.job->>'change' = existing.job->>'change'
+					-- from the same source storage (if applicable, as 'gc' has no source)
+					AND COALESCE(queue.job->>'source_node_storage', '') = COALESCE(existing.job->>'source_node_storage', ''))
+				)
+			)
+			RETURNING queue.id, queue.lock_id
+		)
+		, updated AS (
 			UPDATE replication_queue AS queue
-			SET
-				state = CASE WHEN state = 'in_progress' THEN
-						$2::REPLICATION_JOB_STATE
-					ELSE
-						(CASE WHEN $2 = 'completed' THEN 'completed' ELSE queue.state END)::REPLICATION_JOB_STATE
-					END,
-				updated_at = CASE WHEN state = 'in_progress' THEN
-						NOW() AT TIME ZONE 'UTC'
-					ELSE
-						(CASE WHEN $2 = 'completed' THEN NOW() AT TIME ZONE 'UTC' ELSE queue.updated_at END)
-					END
+			SET state = $2::REPLICATION_JOB_STATE,
+				updated_at = NOW() AT TIME ZONE 'UTC'
 			FROM existing
 			WHERE existing.id = queue.id
-				OR (
-					    queue.state = 'ready'
-					AND queue.created_at < existing.updated_at
-					AND queue.lock_id = existing.lock_id
-					AND queue.job->>'change' = existing.job->>'change'
-					AND queue.job->>'source_node_storage' = existing.job->>'source_node_storage'
-				)
 			RETURNING queue.id, queue.lock_id
 		)
 		, removed_job_lock AS (
 			DELETE FROM replication_queue_job_lock AS job_lock
-			USING to_release
+			USING (SELECT * FROM deleted UNION SELECT * FROM updated) AS to_release
 			WHERE job_lock.job_id = to_release.id AND job_lock.lock_id = to_release.lock_id
 			RETURNING to_release.lock_id
 		)
@@ -334,71 +411,6 @@ func (rq PostgresReplicationEventQueue) Acknowledge(ctx context.Context, state J
 	return acknowledged.Values(), nil
 }
 
-func (rq PostgresReplicationEventQueue) GetOutdatedRepositories(ctx context.Context, virtualStorage, reference string) (map[string][]string, error) {
-	const q = `
-WITH latest_jobs AS (
-	SELECT DISTINCT ON (repository, target)
-		job->>'relative_path' AS repository,
-		job->>'target_node_storage' AS target,
-		job->>'source_node_storage' AS source,
-		state
-	FROM replication_queue
-	WHERE job->>'virtual_storage' = $1 AND
-		job->>'target_node_storage' != $2
-	ORDER BY repository, target, updated_at DESC NULLS FIRST
-)
-
-SELECT repository, target
-FROM latest_jobs
-WHERE state != 'completed' OR source != $2
-ORDER BY repository, target
-`
-
-	rows, err := rq.qc.QueryContext(ctx, q, virtualStorage, reference)
-	if err != nil {
-		return nil, err
-	}
-
-	defer rows.Close()
-
-	nodesByRepo := map[string][]string{}
-	for rows.Next() {
-		var repo, node string
-		if err := rows.Scan(&repo, &node); err != nil {
-			return nil, err
-		}
-
-		nodesByRepo[repo] = append(nodesByRepo[repo], node)
-	}
-
-	return nodesByRepo, rows.Err()
-}
-
-func (rq PostgresReplicationEventQueue) GetUpToDateStorages(ctx context.Context, virtualStorage, repoPath string) ([]string, error) {
-	query := `
-		SELECT storage
-		FROM (
-			SELECT DISTINCT ON (job ->> 'target_node_storage')
-				job ->> 'target_node_storage' AS storage,
-				state
-			FROM replication_queue
-			WHERE job ->> 'virtual_storage' = $1 AND job ->> 'relative_path' = $2
-			ORDER BY job ->> 'target_node_storage', updated_at DESC NULLS FIRST
-		) t
-		WHERE state = 'completed'`
-	rows, err := rq.qc.QueryContext(ctx, query, virtualStorage, repoPath)
-	if err != nil {
-		return nil, fmt.Errorf("query: %w", err)
-	}
-
-	var storages glsql.StringProvider
-	if err := glsql.ScanAll(rows, &storages); err != nil {
-		return nil, fmt.Errorf("scan: %w", err)
-	}
-
-	return storages.Values(), nil
-}
-
 // StartHealthUpdate starts periodical update of the event's health identifier.
 // The events with fresh health identifier won't be considered as stale.
 // The health update will be executed on each new entry received from trigger channel passed in.
