@@ -451,11 +451,12 @@ func TestPostgresReplicationEventQueue_Acknowledge(t *testing.T) {
 
 	event.State = JobStateCompleted
 	event.Attempt = 2
-	requireEvents(t, ctx, db, []ReplicationEvent{event})
+	// events acknowledged with 'completed' or 'dead' states expected to be removed
+	db.RequireRowsInTable(t, "replication_queue", 0)
+	//all associated with acknowledged event tracking bindings between lock and event must be removed
+	db.RequireRowsInTable(t, "replication_queue_job_lock", 0)
 	// lock must be released as the event was acknowledged and there are no other events left protected under this lock
 	requireLocks(t, ctx, db, []LockRow{{ID: event.LockID, Acquired: false}})
-	// all associated with acknowledged event tracking bindings between lock and event must be removed
-	requireJobLocks(t, ctx, db, nil)
 }
 
 func TestPostgresReplicationEventQueue_AcknowledgeMultiple(t *testing.T) {
@@ -581,6 +582,7 @@ func TestPostgresReplicationEventQueue_AcknowledgeMultiple(t *testing.T) {
 	requireJobLocks(t, ctx, db, []JobLockRow{
 		{JobID: events[2].ID, LockID: events[2].LockID},
 	})
+	db.RequireRowsInTable(t, "replication_queue", 4)
 
 	dequeuedEvents3, err := queue.Dequeue(ctx, "praefect", "gitaly-2", 3)
 	require.NoError(t, err)
@@ -606,6 +608,7 @@ func TestPostgresReplicationEventQueue_AcknowledgeMultiple(t *testing.T) {
 		{ID: events[6].LockID, Acquired: false},
 	})
 	requireJobLocks(t, ctx, db, nil)
+	db.RequireRowsInTable(t, "replication_queue", 1)
 
 	newEvent, err := queue.Enqueue(ctx, eventType1)
 	require.NoError(t, err)
@@ -613,6 +616,7 @@ func TestPostgresReplicationEventQueue_AcknowledgeMultiple(t *testing.T) {
 	acknowledge4, err := queue.Acknowledge(ctx, JobStateCompleted, []uint64{newEvent.ID})
 	require.NoError(t, err)
 	require.Equal(t, ([]uint64)(nil), acknowledge4) // event that was not dequeued can't be acknowledged
+	db.RequireRowsInTable(t, "replication_queue", 2)
 
 	var newEventState string
 	require.NoError(t, db.QueryRow("SELECT state FROM replication_queue WHERE id = $1", newEvent.ID).Scan(&newEventState))
@@ -626,224 +630,6 @@ func TestPostgresReplicationEventQueue_AcknowledgeMultiple(t *testing.T) {
 	requireJobLocks(t, ctx, db, nil)
 }
 
-func TestPostgresReplicationEventQueue_GetOutdatedRepositories(t *testing.T) {
-	db := getDB(t)
-	contractTestQueueGetOutdatedRepositories(t,
-		NewPostgresReplicationEventQueue(db),
-		func(t testing.TB, events []ReplicationEvent) {
-			db.TruncateAll(t)
-			for _, event := range events {
-				db.MustExec(t, "INSERT INTO replication_queue (state, updated_at, job) VALUES ($1, $2, $3)",
-					event.State, event.UpdatedAt, event.Job,
-				)
-			}
-		},
-	)
-}
-
-func TestPostgresReplicationEventQueue_GetUpToDateStorages(t *testing.T) {
-	db := getDB(t)
-
-	ctx, cancel := testhelper.Context()
-	defer cancel()
-
-	source := PostgresReplicationEventQueue{qc: db}
-
-	t.Run("single 'ready' job for single storage", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'ready')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{}, ss)
-	})
-
-	t.Run("single 'dead' job for single storage", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'dead')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{}, ss)
-	})
-
-	t.Run("single 'failed' job for single storage", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'failed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{}, ss)
-	})
-
-	t.Run("single 'completed' job for single storage", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s1"}, ss)
-	})
-
-	t.Run("multiple 'completed' jobs for single storage but different repos", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-2"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s1"}, ss)
-	})
-
-	t.Run("last jobs are 'completed' for multiple storages", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s1", "s2"}, ss)
-	})
-
-	t.Run("last jobs are 'completed' for multiple storages but different virtuals", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-				('{"virtual_storage": "vs2", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s1"}, ss)
-	})
-
-	t.Run("lasts are in 'completed' and 'in_progress' for different storages", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'in_progress')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s1"}, ss)
-	})
-
-	t.Run("lasts are in 'dead', 'ready', 'failed' and 'in_progress' for different storages", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'dead'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'ready'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s3", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'failed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s4", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'in_progress')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{}, ss)
-	})
-
-	t.Run("last is not 'completed'", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'dead'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'ready'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s3", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'failed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s3", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s4", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'failed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s4", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{}, ss)
-	})
-
-	t.Run("multiple virtuals with multiple storages", func(t *testing.T) {
-		db.TruncateAll(t)
-
-		db.MustExec(t, `
-			INSERT INTO replication_queue
-				(job, updated_at, state)
-			VALUES
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'dead'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s1", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'completed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s2", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'dead'),
-
-				('{"virtual_storage": "vs2", "target_node_storage": "s3", "relative_path": "path-1"}', '2020-01-01 00:00:01', 'completed'),
-				('{"virtual_storage": "vs2", "target_node_storage": "s3", "relative_path": "path-1"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s4", "relative_path": "path-2"}', '2020-01-01 00:00:01', 'failed'),
-				('{"virtual_storage": "vs1", "target_node_storage": "s4", "relative_path": "path-2"}', '2020-01-01 00:00:00', 'completed'),
-
-				('{"virtual_storage": "vs1", "target_node_storage": "s5", "relative_path": "path-2"}', '2020-01-01 00:00:00', 'completed')`,
-		)
-
-		ss, err := source.GetUpToDateStorages(ctx, "vs1", "path-1")
-		require.NoError(t, err)
-		require.ElementsMatch(t, []string{"s2"}, ss)
-	})
-}
-
 func TestPostgresReplicationEventQueue_StartHealthUpdate(t *testing.T) {
 	db := getDB(t)
 
@@ -1057,10 +843,9 @@ func TestPostgresReplicationEventQueue_AcknowledgeStale(t *testing.T) {
 		require.NoError(t, source.AcknowledgeStale(ctx, time.Microsecond))
 
 		devents2[0].State = JobStateFailed
-		devents3[0].State = JobStateDead
 		devents4[0].Attempt = 2
 		devents4[0].State = JobStateFailed
-		requireEvents(t, ctx, db, []ReplicationEvent{event1, devents2[0], devents3[0], devents4[0]})
+		requireEvents(t, ctx, db, []ReplicationEvent{event1, devents2[0], devents4[0]})
 	})
 
 	t.Run("stale jobs updated for all virtual storages and storages at once", func(t *testing.T) {
