@@ -12,13 +12,14 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/grpc-proxy/proxy"
 	testservice "gitlab.com/gitlab-org/gitaly/internal/praefect/grpc-proxy/testdata"
+	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 )
 
 // TestStreamPeeking demonstrates that a director function is able to peek
 // into a stream. Further more, it demonstrates that peeking into a stream
 // will not disturb the stream sent from the proxy client to the backend.
 func TestStreamPeeking(t *testing.T) {
-	ctx, cancel := context.WithTimeout(context.Background(), 2*time.Second)
+	ctx, cancel := testhelper.Context(testhelper.ContextWithTimeout(2 * time.Second))
 	defer cancel()
 
 	backendCC, backendSrvr, cleanupPinger := newBackendPinger(t, ctx)
@@ -76,7 +77,7 @@ func TestStreamPeeking(t *testing.T) {
 }
 
 func TestStreamInjecting(t *testing.T) {
-	ctx, cancel := context.WithTimeout(context.Background(), 2*time.Second)
+	ctx, cancel := testhelper.Context(testhelper.ContextWithTimeout(2 * time.Second))
 	defer cancel()
 
 	backendCC, backendSrvr, cleanupPinger := newBackendPinger(t, ctx)
