@@ -12,6 +12,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/client"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/datastore"
+	"gitlab.com/gitlab-org/gitaly/internal/praefect/protoregistry"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper/promtest"
 	"google.golang.org/grpc"
@@ -69,7 +70,7 @@ func TestNodeStatus(t *testing.T) {
 	mockHistogramVec := promtest.NewMockHistogramVec()
 
 	storageName := "default"
-	cs := newConnectionStatus(config.Node{Storage: storageName}, cc, testhelper.DiscardTestEntry(t), mockHistogramVec)
+	cs := newConnectionStatus(config.Node{Storage: storageName}, cc, testhelper.DiscardTestEntry(t), mockHistogramVec, nil)
 
 	ctx, cancel := testhelper.Context()
 	defer cancel()
@@ -121,7 +122,7 @@ func TestManagerFailoverDisabledElectionStrategySQL(t *testing.T) {
 		Failover:        config.Failover{Enabled: false, ElectionStrategy: "sql"},
 		VirtualStorages: []*config.VirtualStorage{virtualStorage},
 	}
-	nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 
 	nm.Start(time.Millisecond, time.Millisecond)
@@ -167,7 +168,7 @@ func TestDialWithUnhealthyNode(t *testing.T) {
 	srv, _ := testhelper.NewHealthServerWithListener(t, primaryLn)
 	defer srv.Stop()
 
-	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 
 	mgr.Start(1*time.Millisecond, 1*time.Millisecond)
@@ -213,7 +214,7 @@ func TestGetPrimaries(t *testing.T) {
 	primarySrv, _ := testhelper.NewHealthServerWithListener(t, primaryListener)
 	defer primarySrv.Stop()
 
-	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec())
+	mgr, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 
 	for i := 0; i < healthcheckThreshold; i++ {
@@ -273,10 +274,10 @@ func TestNodeManager(t *testing.T) {
 	}
 
 	mockHistogram := promtest.NewMockHistogramVec()
-	nm, err := NewManager(testhelper.DiscardTestEntry(t), confWithFailover, nil, nil, mockHistogram)
+	nm, err := NewManager(testhelper.DiscardTestEntry(t), confWithFailover, nil, nil, mockHistogram, protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 
-	nmWithoutFailover, err := NewManager(testhelper.DiscardTestEntry(t), confWithoutFailover, nil, nil, mockHistogram)
+	nmWithoutFailover, err := NewManager(testhelper.DiscardTestEntry(t), confWithoutFailover, nil, nil, mockHistogram, protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 
 	// monitoring period set to 1 hour as we execute health checks by hands in this test
@@ -376,7 +377,7 @@ func TestMgr_GetSyncedNode(t *testing.T) {
 		conf.Failover.Enabled = failover
 		rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-		nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, rs, promtest.NewMockHistogramVec())
+		nm, err := NewManager(testhelper.DiscardTestEntry(t), conf, nil, rs, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 		require.NoError(t, err)
 
 		for i := range healthSrvs {
@@ -532,12 +533,12 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	latencyHistMock := &promtest.MockHistogramVec{}
 
 	t.Run("unchecked node is unhealthy", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		require.False(t, ns.IsHealthy())
 	})
 
 	t.Run("not enough check to consider it healthy", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		healthSrv.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 		checkNTimes(ctx, t, ns, healthcheckThreshold-1)
 
@@ -545,7 +546,7 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	})
 
 	t.Run("healthy", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		healthSrv.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 		checkNTimes(ctx, t, ns, healthcheckThreshold)
 
@@ -553,7 +554,7 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	})
 
 	t.Run("healthy turns into unhealthy after single failed check", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		healthSrv.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 		checkNTimes(ctx, t, ns, healthcheckThreshold)
 
@@ -566,7 +567,7 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	})
 
 	t.Run("unhealthy turns into healthy after pre-define threshold of checks", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		healthSrv.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 		checkNTimes(ctx, t, ns, healthcheckThreshold)
 
@@ -588,7 +589,7 @@ func TestNodeStatus_IsHealthy(t *testing.T) {
 	})
 
 	t.Run("concurrent access has no races", func(t *testing.T) {
-		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock)
+		ns := newConnectionStatus(node, clientConn, logger, latencyHistMock, nil)
 		healthSrv.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 
 		t.Run("continuously does health checks - 1", func(t *testing.T) {
