@@ -16,7 +16,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes"
 	prommetrics "gitlab.com/gitlab-org/gitaly/internal/prometheus/metrics"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
-	grpccorrelation "gitlab.com/gitlab-org/labkit/correlation/grpc"
+	"gitlab.com/gitlab-org/labkit/correlation"
 	"golang.org/x/sync/errgroup"
 	"google.golang.org/grpc"
 )
@@ -38,8 +38,8 @@ type Replicator interface {
 }
 
 type defaultReplicator struct {
-	log *logrus.Entry
 	rs  datastore.RepositoryStore
+	log logrus.FieldLogger
 }
 
 func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.ReplicationEvent, sourceCC, targetCC *grpc.ClientConn) error {
@@ -53,11 +53,24 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 		RelativePath: event.Job.RelativePath,
 	}
 
+	logger := dr.log.WithFields(logrus.Fields{
+		logWithVirtualStorage: event.Job.VirtualStorage,
+		logWithReplTarget:     event.Job.TargetNodeStorage,
+		logWithCorrID:         correlation.ExtractFromContext(ctx),
+	})
+
 	generation, err := dr.rs.GetReplicatedGeneration(ctx, event.Job.VirtualStorage, event.Job.RelativePath, event.Job.SourceNodeStorage, event.Job.TargetNodeStorage)
 	if err != nil {
 		// Later generation might have already been replicated by an earlier replication job. If that's the case,
 		// we'll simply acknowledge the job. This also prevents accidental downgrades from happening.
-		if errors.Is(err, datastore.DowngradeAttemptedError{}) {
+		var downgradeErr datastore.DowngradeAttemptedError
+		if errors.As(err, &downgradeErr) {
+			message := "repository downgrade prevented"
+			if downgradeErr.CurrentGeneration == downgradeErr.AttemptedGeneration {
+				message = "target repository already on the same generation, skipping replication job"
+			}
+
+			logger.WithError(downgradeErr).Info(message)
 			return nil
 		}
 
@@ -70,7 +83,7 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 		Source:     sourceRepository,
 		Repository: targetRepository,
 	}); err != nil {
-		return fmt.Errorf("failed to create repository: %v", err)
+		return fmt.Errorf("failed to create repository: %w", err)
 	}
 
 	// check if the repository has an object pool
@@ -97,7 +110,7 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 		}
 	}
 
-	checksumsMatch, err := dr.confirmChecksums(ctx, gitalypb.NewRepositoryServiceClient(sourceCC), targetRepositoryClient, sourceRepository, targetRepository)
+	checksumsMatch, err := dr.confirmChecksums(ctx, logger, gitalypb.NewRepositoryServiceClient(sourceCC), targetRepositoryClient, sourceRepository, targetRepository)
 	if err != nil {
 		return err
 	}
@@ -108,10 +121,8 @@ func (dr defaultReplicator) Replicate(ctx context.Context, event datastore.Repli
 			targetRepository.GetStorageName(),
 			sourceRepository.GetStorageName(),
 		).Inc()
-		dr.log.WithFields(logrus.Fields{
-			"primary": sourceRepository,
-			"replica": targetRepository,
-		}).Error("checksums do not match")
+
+		logger.Error("checksums do not match")
 	}
 
 	if generation != datastore.GenerationUnknown {
@@ -147,7 +158,9 @@ func (dr defaultReplicator) Destroy(ctx context.Context, event datastore.Replica
 			return err
 		}
 
-		dr.log.WithError(err).Info("replicated repository delete does not have a store entry")
+		dr.log.WithField(logWithCorrID, correlation.ExtractFromContext(ctx)).
+			WithError(err).
+			Info("replicated repository delete does not have a store entry")
 	}
 
 	return nil
@@ -187,7 +200,9 @@ func (dr defaultReplicator) Rename(ctx context.Context, event datastore.Replicat
 			return err
 		}
 
-		dr.log.WithError(err).Info("replicated repository rename does not have a store entry")
+		dr.log.WithField(logWithCorrID, correlation.ExtractFromContext(ctx)).
+			WithError(err).
+			Info("replicated repository rename does not have a store entry")
 	}
 
 	return nil
@@ -271,7 +286,7 @@ func getChecksumFunc(ctx context.Context, client gitalypb.RepositoryServiceClien
 	}
 }
 
-func (dr defaultReplicator) confirmChecksums(ctx context.Context, primaryClient, replicaClient gitalypb.RepositoryServiceClient, primary, replica *gitalypb.Repository) (bool, error) {
+func (dr defaultReplicator) confirmChecksums(ctx context.Context, logger logrus.FieldLogger, primaryClient, replicaClient gitalypb.RepositoryServiceClient, primary, replica *gitalypb.Repository) (bool, error) {
 	g, gCtx := errgroup.WithContext(ctx)
 
 	var primaryChecksum, replicaChecksum string
@@ -283,12 +298,10 @@ func (dr defaultReplicator) confirmChecksums(ctx context.Context, primaryClient,
 		return false, err
 	}
 
-	dr.log.WithFields(logrus.Fields{
-		"primary":          primary,
-		"replica":          replica,
+	logger.WithFields(logrus.Fields{
 		"primary_checksum": primaryChecksum,
 		"replica_checksum": replicaChecksum,
-	}).Info("replication finished")
+	}).Info("checksum comparison completed")
 
 	return primaryChecksum == replicaChecksum, nil
 }
@@ -305,8 +318,8 @@ type ReplMgr struct {
 	replDelayMetric    prommetrics.HistogramVec
 	replJobTimeout     time.Duration
 	dequeueBatchSize   uint
-	// whitelist contains the project names of the repos we wish to replicate
-	whitelist map[string]struct{}
+	// allowlist contains the project names of the repos we wish to replicate
+	allowlist map[string]struct{}
 }
 
 // ReplMgrOpt allows a replicator to be configured with additional options
@@ -339,8 +352,8 @@ func NewReplMgr(log *logrus.Entry, virtualStorages []string, queue datastore.Rep
 	r := ReplMgr{
 		log:             log.WithField("component", "replication_manager"),
 		queue:           queue,
-		whitelist:       map[string]struct{}{},
-		replicator:      defaultReplicator{log, rs},
+		allowlist:       map[string]struct{}{},
+		replicator:      defaultReplicator{rs: rs, log: log.WithField("component", "replicator")},
 		virtualStorages: virtualStorages,
 		nodeManager:     nodeMgr,
 		replInFlightMetric: prometheus.NewGaugeVec(
@@ -369,11 +382,11 @@ func (r ReplMgr) Collect(ch chan<- prometheus.Metric) {
 	r.replInFlightMetric.Collect(ch)
 }
 
-// WithWhitelist will configure a whitelist for repos to allow replication
-func WithWhitelist(whitelistedRepos []string) ReplMgrOpt {
+// WithAllowlist will configure a allowlist for repos to allow replication
+func WithAllowlist(allowlistedRepos []string) ReplMgrOpt {
 	return func(r *ReplMgr) {
-		for _, repo := range whitelistedRepos {
-			r.whitelist[repo] = struct{}{}
+		for _, repo := range allowlistedRepos {
+			r.allowlist[repo] = struct{}{}
 		}
 	}
 }
@@ -386,9 +399,9 @@ func WithReplicator(r Replicator) ReplMgrOpt {
 }
 
 const (
-	logWithReplJobID  = "replication_job_id"
-	logWithReplTarget = "replication_job_target"
-	logWithCorrID     = "correlation_id"
+	logWithReplTarget     = "replication_job_target"
+	logWithCorrID         = "correlation_id"
+	logWithVirtualStorage = "virtual_storage"
 )
 
 type backoff func() time.Duration
@@ -459,7 +472,7 @@ func (r ReplMgr) ProcessStale(ctx context.Context, checkPeriod, staleAfter time.
 }
 
 func (r ReplMgr) processBacklog(ctx context.Context, b BackoffFunc, virtualStorage string) {
-	logger := r.log.WithField("virtual_storage", virtualStorage)
+	logger := r.log.WithField(logWithVirtualStorage, virtualStorage)
 	backoff, reset := b()
 
 	logger.Info("processing started")
@@ -482,7 +495,7 @@ func (r ReplMgr) processBacklog(ctx context.Context, b BackoffFunc, virtualStora
 				if !target.IsHealthy() {
 					continue
 				}
-				totalEvents += r.handleNode(ctx, logger, shard, virtualStorage, target)
+				totalEvents += r.handleNode(ctx, shard, virtualStorage, target)
 			}
 		}
 
@@ -500,10 +513,12 @@ func (r ReplMgr) processBacklog(ctx context.Context, b BackoffFunc, virtualStora
 	}
 }
 
-func (r ReplMgr) handleNode(ctx context.Context, logger logrus.FieldLogger, shard nodes.Shard, virtualStorage string, target nodes.Node) int {
+func (r ReplMgr) handleNode(ctx context.Context, shard nodes.Shard, virtualStorage string, target nodes.Node) int {
+	logger := r.log.WithFields(logrus.Fields{logWithVirtualStorage: virtualStorage, logWithReplTarget: target.GetStorage()})
+
 	events, err := r.queue.Dequeue(ctx, virtualStorage, target.GetStorage(), int(r.dequeueBatchSize))
 	if err != nil {
-		logger.WithField(logWithReplTarget, target.GetStorage()).WithError(err).Error("failed to dequeue replication events")
+		logger.WithError(err).Error("failed to dequeue replication events")
 		return 0
 	}
 
@@ -523,13 +538,17 @@ func (r ReplMgr) handleNode(ctx context.Context, logger logrus.FieldLogger, shar
 	for state, eventIDs := range eventIDsByState {
 		ackIDs, err := r.queue.Acknowledge(ctx, state, eventIDs)
 		if err != nil {
-			logger.WithField("state", state).WithField("event_ids", eventIDs).WithError(err).Error("failed to acknowledge replication events")
+			logger.WithFields(logrus.Fields{"state": state, "event_ids": eventIDs}).
+				WithError(err).
+				Error("failed to acknowledge replication events")
 			continue
 		}
 
 		notAckIDs := subtractUint64(ackIDs, eventIDs)
 		if len(notAckIDs) > 0 {
-			logger.WithField("state", state).WithField("event_ids", notAckIDs).WithError(err).Error("replication events were not acknowledged")
+			logger.WithFields(logrus.Fields{"state": state, "event_ids": notAckIDs}).
+				WithError(err).
+				Error("replication events were not acknowledged")
 		}
 	}
 
@@ -556,23 +575,27 @@ func (r ReplMgr) startHealthUpdate(ctx context.Context, logger logrus.FieldLogge
 }
 
 func (r ReplMgr) handleNodeEvent(ctx context.Context, logger logrus.FieldLogger, shard nodes.Shard, target nodes.Node, event datastore.ReplicationEvent) datastore.JobState {
-	logger = logger.WithFields(logrus.Fields{
-		logWithReplJobID: event.ID,
-		logWithCorrID:    getCorrelationID(event.Meta),
-	})
-	logger.Info("processing replication job")
-	err := r.processReplicationEvent(ctx, event, shard, target.GetConnection())
-	if err == nil {
-		return datastore.JobStateCompleted
+	cid := getCorrelationID(event.Meta)
+	ctx = correlation.ContextWithCorrelation(ctx, cid)
+
+	// we want it to be queryable by common `json.correlation_id` filter
+	logger = logger.WithField(logWithCorrID, cid)
+	// we log all details about the event only once before start of the processing
+	logger.WithField("event", event).Info("replication job processing started")
+
+	if err := r.processReplicationEvent(ctx, event, shard, target.GetConnection()); err != nil {
+		newState := datastore.JobStateFailed
+		if event.Attempt <= 0 {
+			newState = datastore.JobStateDead
+		}
+
+		logger.WithError(err).WithField("new_state", newState).Error("replication job processing finished")
+		return newState
 	}
 
-	logger.WithError(err).Error("replication job failed")
-
-	if event.Attempt <= 0 {
-		return datastore.JobStateDead
-	}
-
-	return datastore.JobStateFailed
+	newState := datastore.JobStateCompleted
+	logger.WithField("new_state", newState).Info("replication job processing finished")
+	return newState
 }
 
 func (r ReplMgr) processReplicationEvent(ctx context.Context, event datastore.ReplicationEvent, shard nodes.Shard, targetCC *grpc.ClientConn) error {
@@ -581,23 +604,19 @@ func (r ReplMgr) processReplicationEvent(ctx context.Context, event datastore.Re
 		return fmt.Errorf("get source node: %w", err)
 	}
 
-	cid := getCorrelationID(event.Meta)
-
-	var replCtx context.Context
 	var cancel func()
 
 	if r.replJobTimeout > 0 {
-		replCtx, cancel = context.WithTimeout(ctx, r.replJobTimeout)
+		ctx, cancel = context.WithTimeout(ctx, r.replJobTimeout)
 	} else {
-		replCtx, cancel = context.WithCancel(ctx)
+		ctx, cancel = context.WithCancel(ctx)
 	}
 	defer cancel()
 
-	injectedCtx, err := helper.InjectGitalyServers(replCtx, event.Job.SourceNodeStorage, source.GetAddress(), source.GetToken())
+	ctx, err = helper.InjectGitalyServers(ctx, event.Job.SourceNodeStorage, source.GetAddress(), source.GetToken())
 	if err != nil {
 		return fmt.Errorf("inject Gitaly servers into context: %w", err)
 	}
-	injectedCtx = grpccorrelation.InjectToOutgoingContext(injectedCtx, cid)
 
 	replStart := time.Now()
 
@@ -609,17 +628,17 @@ func (r ReplMgr) processReplicationEvent(ctx context.Context, event datastore.Re
 
 	switch event.Job.Change {
 	case datastore.UpdateRepo:
-		err = r.replicator.Replicate(injectedCtx, event, source.GetConnection(), targetCC)
+		err = r.replicator.Replicate(ctx, event, source.GetConnection(), targetCC)
 	case datastore.DeleteRepo:
-		err = r.replicator.Destroy(injectedCtx, event, targetCC)
+		err = r.replicator.Destroy(ctx, event, targetCC)
 	case datastore.RenameRepo:
-		err = r.replicator.Rename(injectedCtx, event, targetCC)
+		err = r.replicator.Rename(ctx, event, targetCC)
 	case datastore.GarbageCollect:
-		err = r.replicator.GarbageCollect(injectedCtx, event, targetCC)
+		err = r.replicator.GarbageCollect(ctx, event, targetCC)
 	case datastore.RepackFull:
-		err = r.replicator.RepackFull(injectedCtx, event, targetCC)
+		err = r.replicator.RepackFull(ctx, event, targetCC)
 	case datastore.RepackIncremental:
-		err = r.replicator.RepackIncremental(injectedCtx, event, targetCC)
+		err = r.replicator.RepackIncremental(ctx, event, targetCC)
 	default:
 		err = fmt.Errorf("unknown replication change type encountered: %q", event.Job.Change)
 	}
