@@ -19,26 +19,27 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	gitalyauth "gitlab.com/gitlab-org/gitaly/auth"
-	gitaly_config "gitlab.com/gitlab-org/gitaly/internal/config"
 	"gitlab.com/gitlab-org/gitaly/internal/git/objectpool"
+	gitaly_config "gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/hook"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/rubyserver"
+	serverPkg "gitlab.com/gitlab-org/gitaly/internal/gitaly/server"
+	objectpoolservice "gitlab.com/gitlab-org/gitaly/internal/gitaly/service/objectpool"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/service/ref"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/service/remote"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/service/repository"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/service/ssh"
 	"gitlab.com/gitlab-org/gitaly/internal/middleware/metadatahandler"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/datastore"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/protoregistry"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/transactions"
-	"gitlab.com/gitlab-org/gitaly/internal/rubyserver"
-	serverPkg "gitlab.com/gitlab-org/gitaly/internal/server"
-	"gitlab.com/gitlab-org/gitaly/internal/service/hook"
-	objectpoolservice "gitlab.com/gitlab-org/gitaly/internal/service/objectpool"
-	"gitlab.com/gitlab-org/gitaly/internal/service/ref"
-	"gitlab.com/gitlab-org/gitaly/internal/service/remote"
-	"gitlab.com/gitlab-org/gitaly/internal/service/repository"
-	"gitlab.com/gitlab-org/gitaly/internal/service/ssh"
 	"gitlab.com/gitlab-org/gitaly/internal/storage"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper/promtest"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/labkit/correlation"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/health"
 	healthpb "google.golang.org/grpc/health/grpc_health_v1"
@@ -46,7 +47,7 @@ import (
 	"google.golang.org/grpc/reflection"
 )
 
-func TestProcessReplicationJob(t *testing.T) {
+func TestReplMgr_ProcessBacklog(t *testing.T) {
 	backupStorageName := "backup"
 
 	backupDir, err := ioutil.TempDir(testhelper.GitlabTestStoragePath(), backupStorageName)
@@ -92,8 +93,6 @@ func TestProcessReplicationJob(t *testing.T) {
 		},
 	}
 
-	queue := datastore.NewMemoryReplicationEventQueue(conf)
-
 	// create object pool on the source
 	objectPoolPath := testhelper.NewTestObjectPoolName(t)
 	pool, err := objectpool.NewObjectPool(gitaly_config.NewLocator(gitaly_config.Config), testRepo.GetStorageName(), objectPoolPath)
@@ -125,7 +124,7 @@ func TestProcessReplicationJob(t *testing.T) {
 
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(1*time.Millisecond, 5*time.Millisecond)
 
@@ -154,27 +153,63 @@ func TestProcessReplicationJob(t *testing.T) {
 		Message: "a commit",
 	})
 
-	replicator := defaultReplicator{
-		log: entry,
-		rs:  datastore.NewMemoryRepositoryStore(conf.StorageNames()),
-	}
-
 	var mockReplicationLatencyHistogramVec promtest.MockHistogramVec
 	var mockReplicationDelayHistogramVec promtest.MockHistogramVec
 
+	logger := testhelper.DiscardTestLogger(t)
+	hook := test.NewLocal(logger)
+
+	queue := datastore.NewReplicationEventQueueInterceptor(datastore.NewMemoryReplicationEventQueue(conf))
+	queue.OnAcknowledge(func(ctx context.Context, state datastore.JobState, ids []uint64, queue datastore.ReplicationEventQueue) ([]uint64, error) {
+		cancel() // when it is called we know that replication is finished
+		return queue.Acknowledge(ctx, state, ids)
+	})
+
+	loggerEntry := logger.WithField("test", t.Name())
+	_, err = queue.Enqueue(ctx, events[0])
+	require.NoError(t, err)
+
+	store := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+
 	replMgr := NewReplMgr(
-		testhelper.DiscardTestEntry(t),
+		loggerEntry,
 		conf.VirtualStorageNames(),
 		queue,
-		datastore.NewMemoryRepositoryStore(conf.StorageNames()),
+		store,
 		nodeMgr,
 		WithLatencyMetric(&mockReplicationLatencyHistogramVec),
 		WithDelayMetric(&mockReplicationDelayHistogramVec),
 	)
 
-	replMgr.replicator = replicator
+	replMgr.ProcessBacklog(ctx, ExpBackoffFunc(time.Hour, 0))
 
-	require.NoError(t, replMgr.processReplicationEvent(ctx, events[0], shard, shard.Secondaries[0].GetConnection()))
+	<-ctx.Done()
+
+	logEntries := hook.AllEntries()
+	require.True(t, len(logEntries) > 3, "expected at least 4 log entries to be present")
+	require.Equal(t,
+		[]interface{}{"processing started", "default"},
+		[]interface{}{logEntries[0].Message, logEntries[0].Data["virtual_storage"]},
+	)
+
+	require.Equal(t,
+		[]interface{}{"replication job processing started", "default", "correlation-id"},
+		[]interface{}{logEntries[1].Message, logEntries[1].Data["virtual_storage"], logEntries[1].Data[logWithCorrID]},
+	)
+
+	dequeuedEvent := logEntries[1].Data["event"].(datastore.ReplicationEvent)
+	require.Equal(t, datastore.JobStateInProgress, dequeuedEvent.State)
+	require.Equal(t, []string{"backup", "default"}, []string{dequeuedEvent.Job.TargetNodeStorage, dequeuedEvent.Job.SourceNodeStorage})
+
+	require.Equal(t,
+		[]interface{}{"checksum comparison completed", "default", "correlation-id"},
+		[]interface{}{logEntries[2].Message, logEntries[2].Data["virtual_storage"], logEntries[2].Data[logWithCorrID]},
+	)
+
+	require.Equal(t,
+		[]interface{}{"replication job processing finished", "default", datastore.JobStateCompleted, "correlation-id"},
+		[]interface{}{logEntries[3].Message, logEntries[3].Data["virtual_storage"], logEntries[3].Data["new_state"], logEntries[3].Data[logWithCorrID]},
+	)
 
 	relativeRepoPath, err := filepath.Rel(testhelper.GitlabTestStoragePath(), testRepoPath)
 	require.NoError(t, err)
@@ -193,6 +228,62 @@ gitaly_praefect_replication_jobs{change_type="update",gitaly_storage="backup",vi
 `)))
 }
 
+func TestReplicatorDowngradeAttempt(t *testing.T) {
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	ctx = correlation.ContextWithCorrelation(ctx, "correlation-id")
+
+	rs := datastore.NewMemoryRepositoryStore(nil)
+	require.NoError(t, rs.SetGeneration(ctx, "virtual-storage-1", "relative-path-1", "gitaly-1", 0))
+	require.NoError(t, rs.SetGeneration(ctx, "virtual-storage-1", "relative-path-1", "gitaly-2", 0))
+
+	logger := testhelper.DiscardTestLogger(t)
+	hook := test.NewLocal(logger)
+	r := &defaultReplicator{rs: rs, log: logger}
+
+	require.NoError(t, r.Replicate(ctx, datastore.ReplicationEvent{
+		Job: datastore.ReplicationJob{
+			VirtualStorage:    "virtual-storage-1",
+			RelativePath:      "relative-path-1",
+			SourceNodeStorage: "gitaly-1",
+			TargetNodeStorage: "gitaly-2",
+		},
+	}, nil, nil))
+
+	require.Equal(t, logrus.InfoLevel, hook.LastEntry().Level)
+	require.Equal(t, datastore.DowngradeAttemptedError{
+		VirtualStorage:      "virtual-storage-1",
+		RelativePath:        "relative-path-1",
+		Storage:             "gitaly-2",
+		CurrentGeneration:   0,
+		AttemptedGeneration: 0,
+	}, hook.LastEntry().Data["error"])
+	require.Equal(t, "correlation-id", hook.LastEntry().Data[logWithCorrID])
+	require.Equal(t, "target repository already on the same generation, skipping replication job", hook.LastEntry().Message)
+
+	require.NoError(t, rs.SetGeneration(ctx, "virtual-storage-1", "relative-path-1", "gitaly-2", 1))
+	require.NoError(t, r.Replicate(ctx, datastore.ReplicationEvent{
+		Job: datastore.ReplicationJob{
+			VirtualStorage:    "virtual-storage-1",
+			RelativePath:      "relative-path-1",
+			SourceNodeStorage: "gitaly-1",
+			TargetNodeStorage: "gitaly-2",
+		},
+	}, nil, nil))
+
+	require.Equal(t, logrus.InfoLevel, hook.LastEntry().Level)
+	require.Equal(t, datastore.DowngradeAttemptedError{
+		VirtualStorage:      "virtual-storage-1",
+		RelativePath:        "relative-path-1",
+		Storage:             "gitaly-2",
+		CurrentGeneration:   1,
+		AttemptedGeneration: 0,
+	}, hook.LastEntry().Data["error"])
+	require.Equal(t, "correlation-id", hook.LastEntry().Data[logWithCorrID])
+	require.Equal(t, "repository downgrade prevented", hook.LastEntry().Message)
+}
+
 func TestPropagateReplicationJob(t *testing.T) {
 	primaryServer, primarySocketPath, cleanup := runMockRepositoryServer(t)
 	defer cleanup()
@@ -222,7 +313,7 @@ func TestPropagateReplicationJob(t *testing.T) {
 	queue := datastore.NewMemoryReplicationEventQueue(conf)
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
@@ -230,7 +321,14 @@ func TestPropagateReplicationJob(t *testing.T) {
 
 	rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
 
-	coordinator := NewCoordinator(queue, rs, nodeMgr, txMgr, conf, protoregistry.GitalyProtoPreregistered)
+	coordinator := NewCoordinator(
+		queue,
+		rs,
+		NewNodeManagerRouter(nodeMgr, rs),
+		txMgr,
+		conf,
+		protoregistry.GitalyProtoPreregistered,
+	)
 
 	replmgr := NewReplMgr(logEntry, conf.VirtualStorageNames(), queue, rs, nodeMgr)
 
@@ -389,10 +487,8 @@ func TestConfirmReplication(t *testing.T) {
 	require.NoError(t, err)
 
 	var replicator defaultReplicator
-	entry := testhelper.DiscardTestEntry(t)
-	replicator.log = entry
 
-	equal, err := replicator.confirmChecksums(ctx, gitalypb.NewRepositoryServiceClient(conn), gitalypb.NewRepositoryServiceClient(conn), testRepoA, testRepoB)
+	equal, err := replicator.confirmChecksums(ctx, testhelper.DiscardTestLogger(t), gitalypb.NewRepositoryServiceClient(conn), gitalypb.NewRepositoryServiceClient(conn), testRepoA, testRepoB)
 	require.NoError(t, err)
 	require.True(t, equal)
 
@@ -400,7 +496,7 @@ func TestConfirmReplication(t *testing.T) {
 		Message: "a commit",
 	})
 
-	equal, err = replicator.confirmChecksums(ctx, gitalypb.NewRepositoryServiceClient(conn), gitalypb.NewRepositoryServiceClient(conn), testRepoA, testRepoB)
+	equal, err = replicator.confirmChecksums(ctx, testhelper.DiscardTestLogger(t), gitalypb.NewRepositoryServiceClient(conn), gitalypb.NewRepositoryServiceClient(conn), testRepoA, testRepoB)
 	require.NoError(t, err)
 	require.False(t, equal)
 }
@@ -509,7 +605,7 @@ func TestProcessBacklog_FailedJobs(t *testing.T) {
 
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
@@ -655,7 +751,7 @@ func TestProcessBacklog_Success(t *testing.T) {
 
 	logEntry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(logEntry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
@@ -852,7 +948,7 @@ func TestBackoff(t *testing.T) {
 }
 
 func runFullGitalyServer(t *testing.T) (*grpc.Server, string) {
-	server := serverPkg.NewInsecure(RubyServer, hook.GitlabAPIStub, gitaly_config.Config)
+	server := serverPkg.NewInsecure(RubyServer, hook.NewManager(hook.GitlabAPIStub, gitaly_config.Config), gitaly_config.Config)
 
 	serverSocketPath := testhelper.GetTemporaryGitalySocketFileName()
 
