@@ -16,12 +16,13 @@ import (
 	"time"
 
 	"github.com/golang/protobuf/proto"
+	"github.com/golang/protobuf/ptypes/empty"
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	gconfig "gitlab.com/gitlab-org/gitaly/internal/config"
 	"gitlab.com/gitlab-org/gitaly/internal/git"
+	gconfig "gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
 	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/config"
@@ -30,6 +31,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/metadata"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/mock"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes"
+	"gitlab.com/gitlab-org/gitaly/internal/praefect/nodes/tracker"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/protoregistry"
 	"gitlab.com/gitlab-org/gitaly/internal/praefect/transactions"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
@@ -37,58 +39,13 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/version"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
+	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/health"
 	"google.golang.org/grpc/health/grpc_health_v1"
-	"google.golang.org/grpc/reflection"
+	grpc_metadata "google.golang.org/grpc/metadata"
+	"google.golang.org/grpc/status"
 )
 
-func TestServerRouteServerAccessor(t *testing.T) {
-	var (
-		conf = testConfig(1)
-		reqQ = make(chan *mock.SimpleRequest)
-
-		expectResp = &mock.SimpleResponse{Value: 2}
-
-		// note: a server scoped RPC will be randomly routed
-		// to an available backend server. To simplify our
-		// test, a single backend server is used.
-		backends = map[string]mock.SimpleServiceServer{
-			conf.VirtualStorages[0].Nodes[0].Storage: &mockSvc{
-				serverAccessor: func(_ context.Context, req *mock.SimpleRequest) (*mock.SimpleResponse, error) {
-					reqQ <- req
-					return expectResp, nil
-				},
-			},
-		}
-	)
-
-	cc, _, cleanup := runPraefectServerWithMock(t, conf, nil, backends)
-	defer cleanup()
-
-	cli := mock.NewSimpleServiceClient(cc)
-
-	expectReq := &mock.SimpleRequest{Value: 1}
-
-	done := make(chan struct{})
-	go func() {
-		defer close(done)
-
-		actualReq := <-reqQ
-		assert.True(t, proto.Equal(expectReq, actualReq),
-			"received unexpected request value: %+v instead of %+v", actualReq, expectReq)
-	}()
-
-	ctx, cancel := testhelper.Context(testhelper.ContextWithTimeout(time.Second))
-	defer cancel()
-
-	actualResp, err := cli.ServerAccessor(ctx, expectReq)
-	require.NoError(t, err)
-	require.True(t, proto.Equal(expectResp, actualResp),
-		"expected response was not routed back")
-
-	waitUntil(t, done, time.Second)
-}
-
 func TestGitalyServerInfo(t *testing.T) {
 	gitVersion, err := git.Version()
 	require.NoError(t, err)
@@ -729,7 +686,7 @@ func (m *mockSmartHTTP) Called(method string) int {
 	return m.methodsCalled[method]
 }
 
-func newGrpcServer(t *testing.T, srv gitalypb.SmartHTTPServiceServer) (string, *grpc.Server) {
+func newSmartHTTPGrpcServer(t *testing.T, srv gitalypb.SmartHTTPServiceServer) (string, *grpc.Server) {
 	socketPath := testhelper.GetTemporaryGitalySocketFileName()
 	listener, err := net.Listen("unix", socketPath)
 	require.NoError(t, err)
@@ -740,7 +697,6 @@ func newGrpcServer(t *testing.T, srv gitalypb.SmartHTTPServiceServer) (string, *
 	grpc_health_v1.RegisterHealthServer(grpcServer, healthSrvr)
 	healthSrvr.SetServingStatus("", grpc_health_v1.HealthCheckResponse_SERVING)
 	gitalypb.RegisterSmartHTTPServiceServer(grpcServer, srv)
-	reflection.Register(grpcServer)
 
 	go grpcServer.Serve(listener)
 
@@ -752,11 +708,11 @@ func TestProxyWrites(t *testing.T) {
 
 	smartHTTP0, smartHTTP1, smartHTTP2 := &mockSmartHTTP{txMgr: txMgr}, &mockSmartHTTP{txMgr: txMgr}, &mockSmartHTTP{txMgr: txMgr}
 
-	socket0, srv0 := newGrpcServer(t, smartHTTP0)
+	socket0, srv0 := newSmartHTTPGrpcServer(t, smartHTTP0)
 	defer srv0.Stop()
-	socket1, srv1 := newGrpcServer(t, smartHTTP1)
+	socket1, srv1 := newSmartHTTPGrpcServer(t, smartHTTP1)
 	defer srv1.Stop()
-	socket2, srv2 := newGrpcServer(t, smartHTTP2)
+	socket2, srv2 := newSmartHTTPGrpcServer(t, smartHTTP2)
 	defer srv2.Stop()
 
 	conf := config.Config{
@@ -784,7 +740,7 @@ func TestProxyWrites(t *testing.T) {
 	queue := datastore.NewMemoryReplicationEventQueue(conf)
 	entry := testhelper.DiscardTestEntry(t)
 
-	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec())
+	nodeMgr, err := nodes.NewManager(entry, conf, nil, nil, promtest.NewMockHistogramVec(), protoregistry.GitalyProtoPreregistered, nil)
 	require.NoError(t, err)
 	nodeMgr.Start(0, time.Hour)
 
@@ -804,7 +760,7 @@ func TestProxyWrites(t *testing.T) {
 	coordinator := NewCoordinator(
 		queue,
 		rs,
-		nodeMgr,
+		NewNodeManagerRouter(nodeMgr, rs),
 		txMgr,
 		conf,
 		protoregistry.GitalyProtoPreregistered,
@@ -866,6 +822,167 @@ func TestProxyWrites(t *testing.T) {
 	assert.Equal(t, bytes.Repeat([]byte(payload), 10), receivedData.Bytes())
 }
 
+type errorSimpleService struct{}
+
+// ServerAccessor is implemented by a callback
+func (m *errorSimpleService) ServerAccessor(ctx context.Context, req *mock.SimpleRequest) (*mock.SimpleResponse, error) {
+	return nil, helper.ErrInternalf("something went wrong")
+}
+
+// RepoAccessorUnary is implemented by a callback
+func (m *errorSimpleService) RepoAccessorUnary(ctx context.Context, req *mock.RepoRequest) (*empty.Empty, error) {
+	md, ok := grpc_metadata.FromIncomingContext(ctx)
+	if !ok {
+		return &empty.Empty{}, errors.New("couldn't read metadata")
+	}
+
+	if md.Get("bad-header")[0] == "true" {
+		return &empty.Empty{}, helper.ErrInternalf("something went wrong")
+	}
+
+	return &empty.Empty{}, nil
+}
+
+// RepoMutatorUnary is implemented by a callback
+func (m *errorSimpleService) RepoMutatorUnary(ctx context.Context, req *mock.RepoRequest) (*empty.Empty, error) {
+	md, ok := grpc_metadata.FromIncomingContext(ctx)
+	if !ok {
+		return &empty.Empty{}, errors.New("couldn't read metadata")
+	}
+
+	if md.Get("bad-header")[0] == "true" {
+		return &empty.Empty{}, helper.ErrInternalf("something went wrong")
+	}
+
+	return &empty.Empty{}, nil
+}
+
+func TestErrorThreshold(t *testing.T) {
+	simple := &errorSimpleService{}
+
+	backendToken := ""
+	backend, cleanup := newMockDownstream(t, backendToken, simple)
+	defer cleanup()
+
+	conf := config.Config{
+		VirtualStorages: []*config.VirtualStorage{
+			{
+				Name: "default",
+				Nodes: []*config.Node{
+					{
+						Storage: "praefect-internal-0",
+						Address: backend,
+					},
+				},
+			},
+		},
+		Failover: config.Failover{
+			Enabled:          true,
+			ElectionStrategy: "local",
+		},
+	}
+
+	ctx, cancel := testhelper.Context()
+	defer cancel()
+
+	queue := datastore.NewMemoryReplicationEventQueue(conf)
+	entry := testhelper.DiscardTestEntry(t)
+
+	testCases := []struct {
+		desc     string
+		accessor bool
+	}{
+		{
+			desc:     "read threshold reached",
+			accessor: true,
+		},
+		{
+			desc: "write threshold reached",
+		},
+	}
+
+	gz := proto.FileDescriptor("mock.proto")
+	fd, err := protoregistry.ExtractFileDescriptor(gz)
+	require.NoError(t, err)
+
+	registry, err := protoregistry.New(fd)
+	require.NoError(t, err)
+
+	for _, tc := range testCases {
+		t.Run(tc.desc, func(t *testing.T) {
+			readThreshold := uint32(5000)
+			writeThreshold := uint32(5)
+			if tc.accessor {
+				readThreshold = 5
+				writeThreshold = 5000
+			}
+
+			errorTracker, err := tracker.NewErrors(ctx, 10*time.Second, readThreshold, writeThreshold)
+			require.NoError(t, err)
+
+			rs := datastore.NewMemoryRepositoryStore(conf.StorageNames())
+
+			nodeMgr, err := nodes.NewManager(entry, conf, nil, rs, promtest.NewMockHistogramVec(), registry, errorTracker)
+			require.NoError(t, err)
+
+			coordinator := NewCoordinator(
+				queue,
+				rs,
+				NewNodeManagerRouter(nodeMgr, rs),
+				nil,
+				conf,
+				registry,
+			)
+
+			server := grpc.NewServer(
+				grpc.CustomCodec(proxy.NewCodec()),
+				grpc.UnknownServiceHandler(proxy.TransparentHandler(coordinator.StreamDirector)),
+			)
+
+			socket := testhelper.GetTemporaryGitalySocketFileName()
+			listener, err := net.Listen("unix", socket)
+			require.NoError(t, err)
+
+			go server.Serve(listener)
+			defer server.Stop()
+
+			conn, err := dial("unix://"+socket, []grpc.DialOption{grpc.WithInsecure()})
+			require.NoError(t, err)
+			cli := mock.NewSimpleServiceClient(conn)
+
+			nodeMgr.Start(0, time.Nanosecond)
+			repo, _, cleanup := testhelper.NewTestRepo(t)
+			defer cleanup()
+
+			shard, err := nodeMgr.GetShard("default")
+			require.NoError(t, err)
+			require.Equal(t, "praefect-internal-0", shard.Primary.GetStorage())
+
+			expectedErr := status.Error(codes.Internal, "something went wrong")
+
+			for i := 0; i < 5; i++ {
+				ctx := grpc_metadata.AppendToOutgoingContext(ctx, "bad-header", "true")
+
+				handler := cli.RepoMutatorUnary
+				if tc.accessor {
+					handler = cli.RepoAccessorUnary
+				}
+
+				_, err := handler(ctx, &mock.RepoRequest{Repo: repo})
+				require.Equal(t, expectedErr, err)
+			}
+
+			for i := 0; i < 50; i++ {
+				if _, err = nodeMgr.GetShard("default"); err != nil {
+					break
+				}
+				time.Sleep(1 * time.Millisecond)
+			}
+			require.Equal(t, nodes.ErrPrimaryNotHealthy, err)
+		})
+	}
+}
+
 func newSmartHTTPClient(t *testing.T, serverSocketPath string) (gitalypb.SmartHTTPServiceClient, *grpc.ClientConn) {
 	t.Helper()
 
