@@ -5,11 +5,12 @@ import (
 	"fmt"
 	"os"
 	"os/exec"
-	"path"
+	"path/filepath"
 	"strings"
 	"testing"
 
 	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/gitaly/internal/command"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
 )
 
@@ -63,12 +64,12 @@ func CreateCommit(t testing.TB, repoPath, branchName string, opts *CreateCommitO
 // objects will live in an alternate objects directory. It returns the current
 // head after the command is run and the alternate objects directory path
 func CreateCommitInAlternateObjectDirectory(t testing.TB, repoPath, altObjectsDir string, cmd *exec.Cmd) (currentHead []byte) {
-	gitPath := path.Join(repoPath, ".git")
+	gitPath := filepath.Join(repoPath, ".git")
 
-	altObjectsPath := path.Join(gitPath, altObjectsDir)
+	altObjectsPath := filepath.Join(gitPath, altObjectsDir)
 	gitObjectEnv := []string{
 		fmt.Sprintf("GIT_OBJECT_DIRECTORY=%s", altObjectsPath),
-		fmt.Sprintf("GIT_ALTERNATE_OBJECT_DIRECTORIES=%s", path.Join(gitPath, "objects")),
+		fmt.Sprintf("GIT_ALTERNATE_OBJECT_DIRECTORIES=%s", filepath.Join(gitPath, "objects")),
 	}
 	require.NoError(t, os.MkdirAll(altObjectsPath, 0755))
 
@@ -80,7 +81,7 @@ func CreateCommitInAlternateObjectDirectory(t testing.TB, repoPath, altObjectsDi
 		t.Fatalf("stdout: %s, stderr: %s", output, stderr)
 	}
 
-	cmd = exec.Command("git", "-C", repoPath, "rev-parse", "HEAD")
+	cmd = exec.Command(command.GitPath(), "-C", repoPath, "rev-parse", "HEAD")
 	cmd.Env = gitObjectEnv
 	currentHead, err := cmd.Output()
 	require.NoError(t, err)
