@@ -4,13 +4,12 @@ import (
 	"io/ioutil"
 	"log"
 	"os"
-	"path"
 	"path/filepath"
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/internal/config"
 	"gitlab.com/gitlab-org/gitaly/internal/git/hooks"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
 )
 
 // CaptureHookEnv creates a bogus 'update' Git hook to sniff out what
@@ -45,7 +44,7 @@ func ConfigureGitalyHooksBinary() {
 	goBuildArgs := []string{
 		"build",
 		"-o",
-		path.Join(config.Config.BinDir, "gitaly-hooks"),
+		filepath.Join(config.Config.BinDir, "gitaly-hooks"),
 		"gitlab.com/gitlab-org/gitaly/cmd/gitaly-hooks",
 	}
 	MustRunCommand(nil, nil, "go", goBuildArgs...)
