@@ -18,7 +18,6 @@ import (
 	"net"
 	"os"
 	"os/exec"
-	"path"
 	"path/filepath"
 	"runtime"
 	"sort"
@@ -36,7 +35,7 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"gitlab.com/gitlab-org/gitaly/internal/command"
-	"gitlab.com/gitlab-org/gitaly/internal/config"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/fieldextractors"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
 	gitalylog "gitlab.com/gitlab-org/gitaly/internal/log"
@@ -57,6 +56,7 @@ const (
 	testGitEnv          = "testdata/git-env"
 	GlRepository        = "project-1"
 	GlID                = "user-123"
+	GlProjectPath       = "gitlab-org/gitlab-test"
 )
 
 var configureOnce sync.Once
@@ -126,7 +126,7 @@ func GitlabTestStoragePath() string {
 	if !ok {
 		log.Fatal("Could not get caller info")
 	}
-	return path.Join(path.Dir(currentFile), "testdata/data")
+	return filepath.Join(filepath.Dir(currentFile), "testdata", "data")
 }
 
 // GitalyServersMetadata returns a metadata pair for gitaly-servers to be used in
@@ -216,15 +216,17 @@ func MustRunCommand(t testing.TB, stdin io.Reader, name string, args ...string)
 		t.Helper()
 	}
 
-	cmd := exec.Command(name, args...)
-
+	var cmd *exec.Cmd
 	if name == "git" {
+		cmd = exec.Command(command.GitPath(), args...)
 		cmd.Env = os.Environ()
 		cmd.Env = append(command.GitEnv, cmd.Env...)
 		cmd.Env = append(cmd.Env,
 			"GIT_AUTHOR_DATE=1572776879 +0100",
 			"GIT_COMMITTER_DATE=1572776879 +0100",
 		)
+	} else {
+		cmd = exec.Command(name, args...)
 	}
 
 	if stdin != nil {
@@ -499,9 +501,10 @@ func Context(opts ...ContextOpt) (context.Context, func()) {
 func CreateRepo(t testing.TB, storagePath, relativePath string) *gitalypb.Repository {
 	require.NoError(t, os.MkdirAll(filepath.Dir(storagePath), 0755), "making repo parent dir")
 	return &gitalypb.Repository{
-		StorageName:  "default",
-		RelativePath: relativePath,
-		GlRepository: GlRepository,
+		StorageName:   "default",
+		RelativePath:  relativePath,
+		GlRepository:  GlRepository,
+		GlProjectPath: GlProjectPath,
 	}
 }
 
@@ -529,7 +532,7 @@ func initRepo(t testing.TB, bare bool) (*gitalypb.Repository, string, func()) {
 
 	repo := CreateRepo(t, storagePath, relativePath)
 	if !bare {
-		repo.RelativePath = path.Join(repo.RelativePath, ".git")
+		repo.RelativePath = filepath.Join(repo.RelativePath, ".git")
 	}
 
 	return repo, repoPath, func() { require.NoError(t, os.RemoveAll(repoPath)) }
@@ -572,7 +575,7 @@ func cloneTestRepo(t testing.TB, storageRoot, relativePath string, bare bool) (r
 		args = append(args, "--bare")
 	} else {
 		// For non-bare repos the relative path is the .git folder inside the path
-		repo.RelativePath = path.Join(relativePath, ".git")
+		repo.RelativePath = filepath.Join(relativePath, ".git")
 	}
 
 	MustRunCommand(t, nil, "git", append(args, testRepositoryPath(t), repoPath)...)
@@ -600,12 +603,27 @@ func ConfigureGitalySSH() {
 	goBuildArgs := []string{
 		"build",
 		"-o",
-		path.Join(config.Config.BinDir, "gitaly-ssh"),
+		filepath.Join(config.Config.BinDir, "gitaly-ssh"),
 		"gitlab.com/gitlab-org/gitaly/cmd/gitaly-ssh",
 	}
 	MustRunCommand(nil, nil, "go", goBuildArgs...)
 }
 
+// ConfigureGitalyGit2Go configures the gitaly-git2go command for tests
+func ConfigureGitalyGit2Go() {
+	if config.Config.BinDir == "" {
+		log.Fatal("config.Config.BinDir must be set")
+	}
+
+	goBuildArgs := []string{
+		"build",
+		"-tags", "static,system_libgit2",
+		"-o", filepath.Join(config.Config.BinDir, "gitaly-git2go"),
+		"gitlab.com/gitlab-org/gitaly/cmd/gitaly-git2go",
+	}
+	MustRunCommand(nil, nil, "go", goBuildArgs...)
+}
+
 // GetRepositoryRefs gives a list of each repository ref as a string
 func GetRepositoryRefs(t testing.TB, repoPath string) string {
 	refs := MustRunCommand(t, nil, "git", "-C", repoPath, "for-each-ref")
@@ -661,7 +679,7 @@ func TempDir(t testing.TB) (string, func()) {
 		log.Fatal("Could not get caller info")
 	}
 
-	rootTmpDir := path.Join(path.Dir(currentFile), "testdata/tmp")
+	rootTmpDir := filepath.Join(filepath.Dir(currentFile), "testdata", "tmp")
 	tmpDir, err := ioutil.TempDir(rootTmpDir, "")
 	require.NoError(t, err)
 	return tmpDir, func() { require.NoError(t, os.RemoveAll(tmpDir)) }
@@ -678,7 +696,7 @@ func GitObjectMustNotExist(t testing.TB, repoPath, sha string) {
 }
 
 func gitObjectExists(t testing.TB, repoPath, sha string, exists bool) {
-	cmd := exec.Command("git", "-C", repoPath, "cat-file", "-e", sha)
+	cmd := exec.Command(command.GitPath(), "-C", repoPath, "cat-file", "-e", sha)
 	cmd.Env = []string{
 		"GIT_ALLOW_PROTOCOL=", // To prevent partial clone reaching remote repo over SSH
 	}
@@ -762,17 +780,26 @@ func WriteExecutable(path string, content []byte) (func(), error) {
 	}, ioutil.WriteFile(path, content, 0755)
 }
 
-// CheckNewObjectExists is a script meant to be used as a pre-receive custom hook.
-// It only succeeds if it can find the object in the quarantine directory.
-// if GIT_OBJECT_DIRECTORY and GIT_ALTERNATE_OBJECT_DIRECTORIES were not passed through correctly to the hooks,
-// it will fail
-const CheckNewObjectExists = `#!/usr/bin/env ruby
+// WriteCheckNewObjectExistsHook writes a pre-receive hook which only succeeds
+// if it can find the object in the quarantine directory. if
+// GIT_OBJECT_DIRECTORY and GIT_ALTERNATE_OBJECT_DIRECTORIES were not passed
+// through correctly to the hooks, it will fail
+func WriteCheckNewObjectExistsHook(t *testing.T, repoPath string) func() {
+	hook := fmt.Sprintf(`#!/usr/bin/env ruby
 STDIN.each_line do |line|
   new_object = line.split(' ')[1]
   exit 1 unless new_object
-  exit 1 unless	system(*%W[git cat-file -e #{new_object}])
+  exit 1 unless	system(*%%W[%s cat-file -e #{new_object}])
 end
-`
+`, command.GitPath())
+
+	ioutil.WriteFile("/tmp/file", []byte(hook), 0644)
+
+	cleanup, err := WriteCustomHook(repoPath, "pre-receive", []byte(hook))
+	require.NoError(t, err)
+
+	return cleanup
+}
 
 func WriteBlobs(t testing.TB, testRepoPath string, n int) []string {
 	var blobIDs []string
