@@ -27,14 +27,15 @@ import (
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/require"
 	gitalyauth "gitlab.com/gitlab-org/gitaly/auth"
-	"gitlab.com/gitlab-org/gitaly/internal/config"
-	"gitlab.com/gitlab-org/gitaly/internal/config/auth"
 	"gitlab.com/gitlab-org/gitaly/internal/git/hooks"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config"
+	"gitlab.com/gitlab-org/gitaly/internal/gitaly/config/auth"
+	serverauth "gitlab.com/gitlab-org/gitaly/internal/gitaly/server/auth"
 	"gitlab.com/gitlab-org/gitaly/internal/gitlabshell"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/fieldextractors"
 	praefectconfig "gitlab.com/gitlab-org/gitaly/internal/praefect/config"
-	serverauth "gitlab.com/gitlab-org/gitaly/internal/server/auth"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
+	grpccorrelation "gitlab.com/gitlab-org/labkit/correlation/grpc"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/health"
 	healthpb "google.golang.org/grpc/health/grpc_health_v1"
@@ -298,11 +299,18 @@ func NewServer(tb testing.TB, streamInterceptors []grpc.StreamServerInterceptor,
 	logrusEntry := log.NewEntry(logger).WithField("test", tb.Name())
 
 	ctxTagger := grpc_ctxtags.WithFieldExtractorForInitialReq(fieldextractors.FieldExtractor)
-	ctxStreamTagger := grpc_ctxtags.StreamServerInterceptor(ctxTagger)
-	ctxUnaryTagger := grpc_ctxtags.UnaryServerInterceptor(ctxTagger)
 
-	streamInterceptors = append([]grpc.StreamServerInterceptor{ctxStreamTagger, grpc_logrus.StreamServerInterceptor(logrusEntry)}, streamInterceptors...)
-	unaryInterceptors = append([]grpc.UnaryServerInterceptor{ctxUnaryTagger, grpc_logrus.UnaryServerInterceptor(logrusEntry)}, unaryInterceptors...)
+	streamInterceptors = append([]grpc.StreamServerInterceptor{
+		grpc_ctxtags.StreamServerInterceptor(ctxTagger),
+		grpccorrelation.StreamServerCorrelationInterceptor(),
+		grpc_logrus.StreamServerInterceptor(logrusEntry),
+	}, streamInterceptors...)
+
+	unaryInterceptors = append([]grpc.UnaryServerInterceptor{
+		grpc_ctxtags.UnaryServerInterceptor(ctxTagger),
+		grpccorrelation.UnaryServerCorrelationInterceptor(),
+		grpc_logrus.UnaryServerInterceptor(logrusEntry),
+	}, unaryInterceptors...)
 
 	return NewTestServer(
 		grpc.NewServer(
@@ -331,13 +339,21 @@ func preReceiveFormToMap(u url.Values) map[string]string {
 	}
 }
 
-func postReceiveFormToMap(u url.Values) map[string]interface{} {
-	return map[string]interface{}{
-		"gl_repository": u.Get("gl_repository"),
-		"secret_token":  u.Get("secret_token"),
-		"changes":       u.Get("changes"),
-		"identifier":    u.Get("identifier"),
-		"push_options":  u["push_options[]"],
+type postReceiveForm struct {
+	GLRepository   string   `json:"gl_repository"`
+	SecretToken    string   `json:"secret_token"`
+	Changes        string   `json:"changes"`
+	Identifier     string   `json:"identifier"`
+	GitPushOptions []string `json:"push_options"`
+}
+
+func parsePostReceiveForm(u url.Values) postReceiveForm {
+	return postReceiveForm{
+		GLRepository:   u.Get("gl_repository"),
+		SecretToken:    u.Get("secret_token"),
+		Changes:        u.Get("changes"),
+		Identifier:     u.Get("identifier"),
+		GitPushOptions: u["push_options[]"],
 	}
 }
 
@@ -505,7 +521,7 @@ func handleAllowed(options GitlabTestServerOptions) func(w http.ResponseWriter,
 			return
 		}
 		w.WriteHeader(http.StatusUnauthorized)
-		w.Write([]byte(`{"message":"401 Unauthorized"}`))
+		w.Write([]byte(`{"message":"401 Unauthorized\n"}`))
 	}
 }
 
@@ -600,11 +616,11 @@ func handlePostReceive(options GitlabTestServerOptions) func(w http.ResponseWrit
 			return
 		}
 
-		var params map[string]interface{}
+		var params postReceiveForm
 
 		switch r.Header.Get("Content-Type") {
 		case "application/x-www-form-urlencoded":
-			params = postReceiveFormToMap(r.Form)
+			params = parsePostReceiveForm(r.Form)
 		case "application/json":
 			if err := json.NewDecoder(r.Body).Decode(&params); err != nil {
 				http.Error(w, "could not parse json body", http.StatusBadRequest)
@@ -612,19 +628,19 @@ func handlePostReceive(options GitlabTestServerOptions) func(w http.ResponseWrit
 			}
 		}
 
-		if params["gl_repository"] == "" {
+		if params.GLRepository == "" {
 			http.Error(w, "gl_repository is empty", http.StatusUnauthorized)
 			return
 		}
 
 		if options.GLRepository != "" {
-			if params["gl_repository"] != options.GLRepository {
+			if params.GLRepository != options.GLRepository {
 				http.Error(w, "gl_repository is invalid", http.StatusUnauthorized)
 				return
 			}
 		}
 
-		if params["secret_token"] != options.SecretToken {
+		if params.SecretToken != options.SecretToken {
 			decodedSecret, err := base64.StdEncoding.DecodeString(r.Header.Get("Gitlab-Shared-Secret"))
 			if err != nil {
 				http.Error(w, "secret_token is invalid", http.StatusUnauthorized)
@@ -637,42 +653,39 @@ func handlePostReceive(options GitlabTestServerOptions) func(w http.ResponseWrit
 			}
 		}
 
-		if params["identifier"] == "" {
+		if params.Identifier == "" {
 			http.Error(w, "identifier is empty", http.StatusUnauthorized)
 			return
 		}
 
 		if options.GLRepository != "" {
-			if params["identifier"] != options.GLID {
+			if params.Identifier != options.GLID {
 				http.Error(w, "identifier is invalid", http.StatusUnauthorized)
 				return
 			}
 		}
 
-		if params["changes"] == "" {
+		if params.Changes == "" {
 			http.Error(w, "changes is empty", http.StatusUnauthorized)
 			return
 		}
 
 		if options.Changes != "" {
-			if params["changes"] != options.Changes {
+			if params.Changes != options.Changes {
 				http.Error(w, "changes is invalid", http.StatusUnauthorized)
 				return
 			}
 		}
 
-		if len(options.GitPushOptions) > 0 {
-			pushOptions := params["push_options"].([]string)
-			if len(pushOptions) != len(options.GitPushOptions) {
-				http.Error(w, "git push options is invalid", http.StatusUnauthorized)
-				return
-			}
+		if len(options.GitPushOptions) != len(params.GitPushOptions) {
+			http.Error(w, "invalid push options", http.StatusUnauthorized)
+			return
+		}
 
-			for i, pushOption := range pushOptions {
-				if pushOption != options.GitPushOptions[i] {
-					http.Error(w, "git push options is invalid", http.StatusUnauthorized)
-					return
-				}
+		for i, opt := range options.GitPushOptions {
+			if opt != params.GitPushOptions[i] {
+				http.Error(w, "invalid push options", http.StatusUnauthorized)
+				return
 			}
 		}
 
@@ -739,17 +752,53 @@ type GitlabTestServerOptions struct {
 	GitObjectDir                string
 	GitAlternateObjectDirs      []string
 	RepoPath                    string
+	UnixSocket                  bool
+	RelativeURLRoot             string
 }
 
 // NewGitlabTestServer returns a mock gitlab server that responds to the hook api endpoints
-func NewGitlabTestServer(options GitlabTestServerOptions) *httptest.Server {
+func NewGitlabTestServer(t FatalLogger, options GitlabTestServerOptions) (url string, cleanup func()) {
 	mux := http.NewServeMux()
-	mux.Handle("/api/v4/internal/allowed", http.HandlerFunc(handleAllowed(options)))
-	mux.Handle("/api/v4/internal/pre_receive", http.HandlerFunc(handlePreReceive(options)))
-	mux.Handle("/api/v4/internal/post_receive", http.HandlerFunc(handlePostReceive(options)))
-	mux.Handle("/api/v4/internal/check", http.HandlerFunc(handleCheck(options)))
+	prefix := strings.TrimRight(options.RelativeURLRoot, "/") + "/api/v4/internal"
+	mux.Handle(prefix+"/allowed", http.HandlerFunc(handleAllowed(options)))
+	mux.Handle(prefix+"/pre_receive", http.HandlerFunc(handlePreReceive(options)))
+	mux.Handle(prefix+"/post_receive", http.HandlerFunc(handlePostReceive(options)))
+	mux.Handle(prefix+"/check", http.HandlerFunc(handleCheck(options)))
 
-	return httptest.NewServer(mux)
+	if options.UnixSocket {
+		return startSocketHTTPServer(t, mux)
+	} else {
+		server := httptest.NewServer(mux)
+		return server.URL, server.Close
+	}
+}
+
+func startSocketHTTPServer(t FatalLogger, mux *http.ServeMux) (string, func()) {
+	tmpFile, err := ioutil.TempFile(os.TempDir(), "http-test-server")
+	if err != nil {
+		t.Fatalf("Cannot create temporary file", err)
+	}
+	filename := tmpFile.Name()
+	tmpFile.Close()
+	os.Remove(filename)
+
+	socketListener, err := net.Listen("unix", filename)
+	if err != nil {
+		t.Fatalf("Cannot listen to socket", err)
+	}
+
+	server := http.Server{
+		Handler: mux,
+	}
+
+	go server.Serve(socketListener)
+
+	url := "http+unix://" + filename
+	cleanup := func() {
+		server.Close()
+	}
+
+	return url, cleanup
 }
 
 // CreateTemporaryGitlabShellDir creates a temporary gitlab shell directory. It returns the path to the directory
@@ -902,12 +951,10 @@ func NewHealthServerWithListener(t testing.TB, listener net.Listener) (*grpc.Ser
 }
 
 func SetupAndStartGitlabServer(t FatalLogger, c *GitlabTestServerOptions) (string, func()) {
-	ts := NewGitlabTestServer(*c)
+	url, cleanup := NewGitlabTestServer(t, *c)
 
-	WriteTemporaryGitlabShellConfigFile(t, config.Config.GitlabShell.Dir, GitlabShellConfig{GitlabURL: ts.URL})
+	WriteTemporaryGitlabShellConfigFile(t, config.Config.GitlabShell.Dir, GitlabShellConfig{GitlabURL: url})
 	WriteShellSecretFile(t, config.Config.GitlabShell.Dir, c.SecretToken)
 
-	return ts.URL, func() {
-		ts.Close()
-	}
+	return url, cleanup
 }
