@@ -13,7 +13,7 @@ module Gitlab
         @push_options = push_options
         @transaction = transaction
 
-        %w[pre-receive update].each do |hook_name|
+        %w[pre-receive update reference-transaction].each do |hook_name|
           status, message = run_hook(hook_name)
 
           raise PreReceiveError, message unless status
